/*
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * Test some of the debugging interfaces
 */

#include <stdio.h>
#include <assert.h>
#include <string.h>
#include <pthread.h>

#include <nodeos/an_impl.h>
#include <nodeos/an_flow.h>
#include <nodeos/an_thread.h>
#include <nodeos/an_debug.h>

static void ee_init(void *arg);
an_flowinit_func_t InitEE = { ee_init };

static void expectstr(const char *expected, const char* got) 
{
	if ((expected == NULL) && (got == NULL))
	{
		printf("PASS: Expected <null> and got it.\n");
	}
	else if ((expected == NULL) && (got != NULL))
	{
		printf("FAIL: Expected <null>, but got `%s'.\n", got);
	}
	else if ((expected != NULL) && (got == NULL))
	{
		printf("FAIL: Expected `%s', but got <null>.\n", expected);
	}
	else
	{
		assert(expected);
		assert(got);

		if (strcmp(expected,got))
		{
			printf("FAIL: Expected `%s', but got `%s'\n",
			       expected, got);
		}
		else
		{
			printf("PASS: Expected `%s' and got it.\n",
			       expected);
		}
	}
}

int
ee_main(int argc, char **argv)
{
	const char* noopts="No valid options passed to ani_debug_setmaskstr";

	printf("%s: Debug Test EE.\n", argv[0]);

	expectstr(NULL, an_debug_setmask("ALL"));
	expectstr(NULL, an_debug_setmask("~ALL"));
	expectstr(noopts, an_debug_setmask(""));
	expectstr(noopts, an_debug_setmask("|,||,,,|"));
	expectstr(NULL, an_debug_setmask("|,||,,,|ALL"));
	expectstr(NULL, an_debug_setmask("ALL|,||,,,|"));
	expectstr(NULL, an_debug_setmask("ALL|,||,,,|ALL"));
	expectstr("foo", an_debug_setmask("foo"));
	expectstr("foo", an_debug_setmask("|,||,,,|foo"));
	expectstr("foo", an_debug_setmask("ALL|,||,,,|foo"));
	expectstr("foo,ALL|,||,,,|", an_debug_setmask("foo,ALL|,||,,,|"));
	expectstr("foo|,,,|", an_debug_setmask("ALL|,|foo|,,,|"));

	printf("%s: Debug Test EE done.\n", argv[0]);
	return 0;
}



static void
ee_init(void *arg)
{
	an_initargs_t *args = arg;
	int status;

	status = ee_main(args->argc, args->argv);
	an_flow_destroy(an_flow_current());
}
