#
# Copyright (c) 1999, 2000 University of Utah and the Flux Group.
# All rights reserved.
# This file is part of Janos.
# 
# Janos is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License, version 2, as
# published by the Free Software Foundation.
# 
# Janos is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details, in the file
# COPYING.  If that file is not present, write to the Free
# Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
# 02111-1307, USA
# 
# The University of Utah and the Flux Group requests users to
# return to flux-dist@cs.utah.edu any improvements that they make.
#

#
# Assumes EE and EESRCS are defined by "caller".
#
# Also expects SRCDIR, LD, LDFLAGS, CC, CFLAGS

$(EE)_EXE :=	$(EE)
$(EE)_PEXE :=	$(EE)_p
$(EE)_MB :=	$(EE).mb
$(EE)_MBP :=	$(EE)_p.mb

$(EE)_SRCS := 		$(EESRCS)
$(EE)_DEPFILES :=	$(patsubst %.c,%.d,$($(EE)_SRCS))
$(EE)_MBFILES :=	$(MBFILES)

$(EE)_OBJS :=	$(patsubst %.c,%.o,$(notdir $($(EE)_SRCS)))
$(EE)_POBJS :=	$(patsubst %.c,%.po,$(notdir $($(EE)_SRCS)))

$($(EE)_EXE)_LDFLAGS :=		$(EELDFLAGS) $(ARCH_LDFLAGS) $(NODEOS_LIBS) $(ARCH_LIBS)
$($(EE)_PEXE)_LDFLAGS :=	$(EELDFLAGS) $(ARCH_LDFLAGS) $(NODEOS_PLIBS) $(ARCH_PLIBS)

## Build the vanilla executable
$($(EE)_EXE): $($(EE)_OBJS) $(NODEOS_LIB_DEPS)
	$(SILENT_MAKE_INFORM) "linking $@."
	$(LD) $(LDFLAGS) -o $@ \
		$(filter-out %.a,$^) \
		$($(@)_LDFLAGS)

## Build the profiling executable
$($(EE)_PEXE): $($(EE)_POBJS) $(NODEOS_PLIB_DEPS)
	$(SILENT_MAKE_INFORM) "linking $@."
	$(LD) $(LDFLAGS) -o $@ \
		$(filter-out %.a,$^) \
		$($(@)_LDFLAGS)

# Note cannot use gcc script for LD in mkmbimage
$($(EE)_MB): $($(EE)_EXE)
	$(SILENT_MAKE_INFORM) "creating multiboot image $@ from $^."
	LD=ld $(OSKIT_INSTDIR)/bin/mkmbimage -o $@ $^ $($(EE)_MBFILES)

# Note cannot use gcc script for LD in mkmbimage
$($(EE)_MBP): $($(EE)_PEXE)
	$(SILENT_MAKE_INFORM) "creating profiling multiboot image $@ from $^."
	$(STRIP) --strip-debug -o $(^)_stripped $^
	LD=ld $(OSKIT_INSTDIR)/bin/mkmbimage -o $@ \
		$(^)_stripped $(^)_stripped:$@
	rm -f $(^)_stripped

# Note cannot use gcc script for LD in mkmbimage
$(EE)_armimg: $($(EE)_EXE)
	$(SILENT_MAKE_INFORM) "creating ARM multiboot image $@ from $^."
	$(STRIP) --strip-debug -o $(^)_stripped $(^)
	LD=ld $(OSKIT_INSTDIR)/bin/mkofwimage -o $@ $(^)_stripped
	rm -f $(^)_stripped

-include $($(EE)_DEPFILES)

%.debugmake:
	@echo "define_env.mk debugging for EE \"$*\""
	@echo "  sources = $($(*)_SRCS)"
	@echo "  target  = $($(*)_EXE)"
	@echo "  extra LD flags= $($(*)_LDFLAGS)"
	@echo "  debugging = $($(*)_EXE) <- $($(*)_OBJS)"
	@echo "  profiling = $($(*)_PEXE) <- $($(*)_POBJS)"
	@echo "  depend file = .depend.$(*)"

PHONY: $(EE).debugmake gendepend.$(EE)

