/*
 * Copyright (c) 1999, 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * TODO:
 *
 * Fixup the packets emitted by NetHog to be valid ethernet
 * packets sent to a BOGON ethernet address.  (Optionally,
 * send them, as IP packets, to a host on the output network.)
 */

/* Print stats to console rather than ship them across the network */
//#define PRINTSTATS

/*
 * A demo.
 */
#include <stdio.h>
#include <assert.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netinet/in_systm.h>
#include <netinet/if_ether.h>
#include <netinet/ip.h>
#include <netinet/udp.h>

#include <nodeos/an_impl.h>
#include <nodeos/an_flow.h>
#include <nodeos/an_thread.h>
#include <nodeos/an_chan.h>
#include <nodeos/an_mutex.h>
#include <nodeos/an_cond.h>
#include <nodeos/an_pbuf.h>
#include <nodeos/an_info.h>
#include <nodeos/an_event.h>
#include <nodeos/an_time.h>
#include <nodeos/an_moab_setup.h> /* an_moab_shutdown() */

#include <nodeos/an_debug.h>
#include <nodeos/lib/arplib.h>
#include <nodeos/lib/utils.h>
#include <nodeos/lib/pbcache.h>
#include <nodeos/lib/configfile.h>

#include "demo.h"
#include "stat.h"

/* Extras in src/pq/pq.c */
extern void	pq_disable(void), pq_enable(void);

/* Extras in OSKit */
extern void	stride_scheduler_disable(void);
extern void	stride_scheduler_enable(void);


/*
 * These are command line configurable
 */
#define RUNFOR		0	/* run for this many seconds (0 == forever) */

/*
 * These are hacks
 */
#ifdef PRINTSTATS
#define STATFREQ	(5*1000)	/* how often to print stats (ms) */
#else
#define STATFREQ	1000		/* how often to send stats (ms) */
#endif

/*
 * Ports for the experiment. This makes use of the hpfq_udp_tally program
 * from the oskit examples directory to count packets. That program expects
 * packets on a particular port (DEST_PORT), and from numbered ports
 * starting at 10000 (SRCA_PORT, SRCB_PORT). To make all this work using
 * my two machine network at home, I use another set of ports (DSTA_PORT,
 * DSTB_PORT) to send the packets to. I then rewrite the UDP headers so
 * that the outgoing packets look they are supposed to. So:
 *
 *  TBPCX <--> TBPCY <--> TBPCZ
 */

/* Turn this on to have each CPUHOG thread print its id every million+ cycles. */
#undef CPUHOGSTRM_DEBUG

/* Turn this on to have the NETHOG thread print 'X' every 50 thousand or so packets. */
#undef NETHOGSTRM_DEBUG

/*
 * Resource Defaults
 */

/*
 * Default network bandwidth limits (assume 100Mbit/s -> 12,500kbyte/s avail)
 *
 * Note that only the Data Stream and NetHog Stream actually share an
 * outbound link.
 *
 * The CTRL stream is input only and CPUHog doesn't have any outchans.
 */

/*
 * XXX Moab just uses a stride scheduler that allocates relative shares that
 * have nothing to do with the underlying device bps.  Via an_info_getifs, it
 * returns a maximum ticket value that it considers to be the bandwidth of
 * the interface.  We allocate shares of that (currently known to be
 * 100,000,000 on a 100Mbit link).
 */
#define KBPS(k)   ((k) * 1000 * 8)

#define DATASTRM_DOWN_BANDW	(KBPS(7500))	/* ~7,500KBps */
#define DATASTRM_UP_BANDW	(KBPS(500))	/* 500KBps */

#define NETHOGSTRM_BANDW	(KBPS(1500))	/* 1,500KBps */
#define STATSTRM_BANDW		(KBPS(500))	/* 500KBps */
#define CPUHOGSTRM_BANDW	((void)unused)	/* does not send any packets */
#define CTRLSTRM_BANDW		((void)unused)	/* only receives packets */
/* Total is approx 10,000KBps, around 80% of the physical limit. */

/* Default per-flow thread counts, implies a weak relative CPU slice
 * in round-robin scheduler.  (Round-robin implies an equal share
 * divided among the set of runnable threads which is some subset of
 * the total number of threads.)
 */
#define DATASTRM_MAXTHREADS	1
#define CTRLSTRM_MAXTHREADS     1
#define STATSTRM_MAXTHREADS     1
#define NETHOGSTRM_MAXTHREADS	1
#define CPUHOGSTRM_MAXTHREADS   20

/*
 * Default CPU rate for each "threadpool" in the demo.
 * 
 * Currently there is only one threadpool per flow, channels use the
 * resources of their flow.
 *
 * For all the flows (especially the CPUHOGSTRM), the given % should
 * be split among all of its threads!  (And not that each thread gets
 * that %).
 *
 * The given number is a "fraction" of the total CPU cycles available
 * (E.g., 4 means one-quarter of the total, 10 means one-tenth, etc.)
 */
#define DATASTRM_CPUFRAC_FLOW	2	/* 50% */
#define CTRLSTRM_CPUFRAC_FLOW	20      /*  5% */
#define STATSTRM_CPUFRAC_FLOW	20      /*  5% */
#define NETHOGSTRM_CPUFRAC_FLOW	3       /* 33% (avoid making NetHog CPU dependent...) */
#define CPUHOGSTRM_CPUFRAC_FLOW	20      /*  5% */
/* Total is approx 93% of the CPU... I hope that's enough room for Moab */

/*
 * Stack size for threads in the system.  None of the threads have very
 * deep callgraphs, so we're basically at the OSKit-pthread minimum.
 */
#define DATASTRM_THREAD_STACKSIZE	(32*1024)
#define CTRLSTRM_THREAD_STACKSIZE	(32*1024)
#define STATSTRM_THREAD_STACKSIZE	(32*1024)
#define NETHOGSTRM_THREAD_STACKSIZE	(32*1024)
#define CPUHOGSTRM_THREAD_STACKSIZE	(20*1024)

/*
 * Default buffer allocations for each stream.
 * 
 * For DATASTRM, X buffers are allocated for each inchan (of which
 * there are two).
 */
#define DATASTRM_MAXBUFS	500
#define CTRLSTRM_MAXBUFS	10
#define STATSTRM_MAXBUFS	10
#define NETHOGSTRM_MAXBUFS	1500
#define CPUHOGSTRM_MAXBUFS	((void)unused)

/*
 * Size of the packets sent by the NetHog.  For the other streams,
 * the packet size is data dependent.
 */
#define NETHOGSTRM_PKTSIZE	1514	/* ethernet max */
//#define NETHOGSTRM_PKTSIZE	60	/* ethernet min */


/*
 * Lots of goo
 */
static int		runfor = RUNFOR;
static struct in_addr   inif_ip, outif_ip, ctrlif_ip;
static struct in_addr   inif_mask, outif_mask;
static struct in_addr   statdst_ip, statif_ip;
/* XXX only the 'daddr' field of the following ethaddr's is used. */
static an_ethaddrspec_t	inifaddr, outifaddr;
static an_ethaddrspec_t	ctrlifaddr, statifaddr, statdstaddr;
static int		verbose = 0;
static an_infoif_t	aninfo[8];
static int		nif = 8;
static an_infocpu_t	cpuinfo;
static an_flow_t*	initflow;
static int		running;
static int		ip_magic_id = 23; /* XXX "unique" id for each packet */
static an_timespec_t	starttime;
static an_mutex_t	nethog_mux;
static an_cond_t	nethog_cond;

/*
 * This thing describes the data streams and the control/stats streams.
 */
struct stream {
	an_demuxkey_t	*ipdemux, ipdemux_data;
	an_flow_t	*flow, flow_data;
	an_chan_t	*inchan, inchan_data;
	an_chan_t	*outchan, outchan_data;
	void		*incookie, *outcookie, *flowcookie;
};
static struct stream	StreamData[2], StreamCtl, StreamNetHog, StreamCPUHog, StreamStat;

/*
 * This stuff is for controlling the demo via the control channel.
 */
static volatile int	dying     = 0; /* ee_die() fired off */
static volatile int	nethog_on = 0;
static volatile int	cpuhog_on = 0;

/* Forward decls */

static void		ee_die(void *event, void *arg);

static void		setup_data_stream(void);

static void		setup_control_stream(void);

static void		setup_stat_stream(void);
static void		setup_stat_buffers(void);
static an_pbuf_t       *get_stat_pbuf(void);
static void		put_stat_pbuf(an_pbuf_t *pbuf);
static void		teardown_stat_buffers(void);

static void		setup_nethog_stream(void);
static void		setup_nethog_buffers(void);
static an_pbuf_t       *get_nethog_pbuf(void);
static void		put_nethog_pbuf(an_pbuf_t *pbuf);
static void		teardown_nethog_buffers(void);

static void		setup_cpuhog_stream(void);
static void		cpuhog_pause_threads(void);
static void		cpuhog_start_threads(void);
static void		cpuhog_thread_wait(void);

static config_param_t parameters[] = {
	{ "INIF", CONFIG_INT, { i: 1 } },
#define INIF		parameters[0].value.i
	{ "INIFIP", CONFIG_STRING, { "192.168.1.2" } },
#define INIFIP		parameters[1].value.s
	{ "INIFMASK", CONFIG_STRING, { "255.255.255.0" } },
#define INIFMASK	parameters[2].value.s
	{ "OUTIF", CONFIG_INT, { i: 0 } },
#define OUTIF		parameters[3].value.i
	{ "OUTIFIP", CONFIG_STRING, { "192.168.2.1" } },
#define OUTIFIP		parameters[4].value.s
	{ "OUTIFMASK", CONFIG_STRING, { "255.255.255.0" } },
#define OUTIFMASK	parameters[5].value.s
	{ "CTRLIF", CONFIG_INT, { i: 4 } },
#define CTRLIF		parameters[6].value.i
	{ "CTRLIFIP", CONFIG_STRING, { "155.99.214.136" } },
#define CTRLIFIP	parameters[7].value.s
	{ "CTRLPORT", CONFIG_INT, { i: 9999 } },
#define CTRLPORT	parameters[8].value.i
	{ "STATIF", CONFIG_INT, { i: 4 } },
#define STATIF		parameters[9].value.i
	{ "STATIFIP", CONFIG_STRING, { "155.99.214.136" } },
#define STATIFIP	parameters[10].value.s
	{ "STATIFPORT", CONFIG_INT, { i: 8887 } },
#define STATIFPORT	parameters[11].value.i
	{ "STATDSTIP", CONFIG_STRING, { "155.99.214.74" } },
#define STATDSTIP	parameters[12].value.s
	{ "STATDSTPORT", CONFIG_INT, { i: STAT_RECV_PORT } },
#define STATDSTPORT	parameters[13].value.i
};
int numparameters = sizeof(parameters) / sizeof(parameters[0]);

#define FATAL_AN_ERROR(err) fatal_an_error(__FILE__, __FUNCTION__, __LINE__, err)
static void		fatal_an_error(const char* file,
				       const char *func,
				       int line,
				       an_error_t err) __attribute__((__noreturn__));
static void		stat_send(void *event, void *arg);
static void		stat_print(int showtotal);

/*
 * Huh?
 */
static void
ee_init(void *arg)
{
	an_initargs_t *args = arg;
	int ee_main(int argc, char **argv);
	int status;

	status = ee_main(args->argc, args->argv);
}
an_flowinit_func_t InitEE = { ee_init };

int
ee_main(int argc, char **argv)
{
	const char	*bp;
	an_error_t	err;
	
	assert(argc >= 1);

	printf("%p/%p: %s started\n",
	       an_flow_current(), an_thread_current(), argv[0]);

	/* ee_die() needs to kill its parent flow. */
	initflow = an_flow_current();
	
	/*
	 * Parse command line arguments.
	 */

	/*
	 * Parse parameter file.
	 */
	config_parse("demo.config", parameters, numparameters);
	config_dump("DEMO", parameters, numparameters);

	/*
	 * Parse environment vars.
	 *
	 * Number of seconds to wait.
	 */
	if ((bp = getenv("RUNFOR")) != NULL)
		runfor = atoi(bp);

	/*
	 * These are our IP addresses, converted to network order.
	 */
	inet_aton(INIFIP,    &inif_ip);
	inet_aton(INIFMASK,  &inif_mask);
	inet_aton(OUTIFIP,   &outif_ip);
	inet_aton(OUTIFMASK, &outif_mask);
	inet_aton(CTRLIFIP,  &ctrlif_ip);
	inet_aton(STATIFIP,  &statif_ip);

	/*
	 * Must do this before using find_ethaddr().
	 */
	arp_init();

	/*
	 * Find addresses for the interfaces we care about. Note that
	 * my test network at home uses one interface, while on the
	 * testbed I use two interfaces.
	 */
	err = arp_find_ethaddr(CTRLIF, &ctrlifaddr);
	assert(err == 0);
	if (STATIF != CTRLIF) {
		err = arp_find_ethaddr(STATIF, &statifaddr);
		assert(err == 0);
	}
	else
		memcpy(&statifaddr, &ctrlifaddr, sizeof(ctrlifaddr));
	if (INIF != CTRLIF) {
		err = arp_find_ethaddr(INIF, &inifaddr);
		assert(err == 0);
	}
	else
		memcpy(&inifaddr, &ctrlifaddr, sizeof(ctrlifaddr));
	if (INIF != OUTIF) {
		err = arp_find_ethaddr(OUTIF, &outifaddr);
		assert(err == 0);
	}
	else
		memcpy(&outifaddr, &inifaddr, sizeof(inifaddr));
		
	/*
	 * Configure the interfaces to do ARP.
	 */
	err = arp_config_iface(CTRLIF, &ctrlif_ip);
	assert(err == 0);
	if (STATIF != CTRLIF) {
		err = arp_config_iface(STATIF, &statif_ip);
		assert(err == 0);
	}
	if (INIF != CTRLIF) {
		err = arp_config_iface(INIF, &inif_ip);
		assert(err == 0);
	}
	if (INIF != OUTIF) {
		err = arp_config_iface(OUTIF, &outif_ip);
		assert(err == 0);
	}

	/*
	 * Determine info about stats destination
	 */
	inet_aton(STATDSTIP, &statdst_ip);
	err = arp_arpfor(STATIF, &statdst_ip, statdstaddr.daddr);
	assert(err == 0);
	assert((statdstaddr.daddr[0]|statdstaddr.daddr[1]|
		statdstaddr.daddr[2]|statdstaddr.daddr[3]|
		statdstaddr.daddr[4]|statdstaddr.daddr[5]) != 0);

	/*
	 * Need to find the IF names. 
	 */
	err = an_info_getifs(aninfo, &nif);
	if (err != 0)
		FATAL_AN_ERROR(err);
	assert(nif >= INIF && nif >= OUTIF && nif >= CTRLIF);
	assert(aninfo[INIF].type == ANIF_ETH);
	assert(aninfo[INIF].bandwidth == 100000000);
	assert(aninfo[OUTIF].type == ANIF_ETH);
	assert(aninfo[OUTIF].bandwidth == 100000000);
	assert(aninfo[CTRLIF].type == ANIF_ETH);
	assert(aninfo[CTRLIF].bandwidth == 100000000);
	assert(aninfo[STATIF].type == ANIF_ETH);
	assert(aninfo[STATIF].bandwidth == 100000000);

	/*
	 * CPU info.
	 */
	an_info_getcpu(&cpuinfo);
	       
	/*
	 * Put out some generic info
	 */
	printf("INIF:      %s (%02x:%02x:%02x:%02x:%02x:%02x), "
	       "inet %s, netmask %s\n",
	       aninfo[INIF].pname,
	       inifaddr.daddr[0], inifaddr.daddr[1], inifaddr.daddr[2],
	       inifaddr.daddr[3], inifaddr.daddr[4], inifaddr.daddr[5],
	       inet_ntoa(inif_ip), INIFMASK);
	printf("OUTIF:     %s (%02x:%02x:%02x:%02x:%02x:%02x), "
	       "inet %s, netmask %s\n",
	       aninfo[OUTIF].pname,
	       outifaddr.daddr[0], outifaddr.daddr[1], outifaddr.daddr[2],
	       outifaddr.daddr[3], outifaddr.daddr[4], outifaddr.daddr[5],
	       inet_ntoa(outif_ip), OUTIFMASK);
	printf("CTRLIF:    %s (%02x:%02x:%02x:%02x:%02x:%02x), "
	       "IP %s, port %d\n",
	       aninfo[CTRLIF].pname,
	       ctrlifaddr.daddr[0], ctrlifaddr.daddr[1], ctrlifaddr.daddr[2],
	       ctrlifaddr.daddr[3], ctrlifaddr.daddr[4], ctrlifaddr.daddr[5],
	       inet_ntoa(ctrlif_ip), CTRLPORT);
	printf("STATIF:    %s (%02x:%02x:%02x:%02x:%02x:%02x), "
	       "IP %s, port %d\n",
	       aninfo[STATIF].pname,
	       statifaddr.daddr[0], statifaddr.daddr[1], statifaddr.daddr[2],
	       statifaddr.daddr[3], statifaddr.daddr[4], statifaddr.daddr[5],
	       inet_ntoa(statif_ip), STATIFPORT);
	printf("STATDST:   %s (%02x:%02x:%02x:%02x:%02x:%02x), port %d\n",
	       inet_ntoa(statdst_ip),
	       statdstaddr.daddr[0], statdstaddr.daddr[1],
	       statdstaddr.daddr[2], statdstaddr.daddr[3],
	       statdstaddr.daddr[4], statdstaddr.daddr[5],
	       STATDSTPORT);

	/*
	 * Configure the data streams
	 */
	setup_control_stream();
	setup_data_stream();
	setup_stat_stream();
	setup_nethog_stream();
	setup_cpuhog_stream();

	/*
	 * XXX need proper synchronization
	 */
	running = 1;
	an_gettime(&starttime);

	/*
	 * Setup a timer to kill us.
	 */
	if (runfor != 0) {
		/* XXX bogus interface alert: watch for wrap */
		assert(runfor < 4200);

		err = an_event_schedule(ee_die, 0, runfor * 1000000);
		if (err != 0)
			FATAL_AN_ERROR(err);
		printf("Timer started, running for %d seconds\n", runfor);
	} else
		printf("Running forever (send \"exit\" command to stop)\n");

	return 0;
}

/* XXX */
#define ETH_TYPEOFF	(12)		/* ethtype (2 bytes) */
#define ETHIP_IPOFF	(14 +  0)	/* IP version/size (1 byte) */
#define ETHIP_IPDSTOFF	(14 + 16)	/* IP dest addr (4 bytes) */
#define ETHIP_PLOFF	(14 + 20)	/* IP min packet len */

#define ETH_TYPEOFF_REL		(12)		/* from packet start */
#define ETHIP_IPOFF_REL		( 0)		/* from typeoff */
#define ETHIP_IPDSTOFF_REL	(15)		/* from ip start */

/*
 * Data stream IP receive function.
 */
static void
ee_ipData(an_pbuf_t *pbuf, void *arg, int moff)
{
	struct stream	*streamp = arg;
	int		inif, outif;
	int		psize = an_pbuf_size(pbuf);
	void		*data = an_pbuf_data(pbuf);
	an_error_t	err;
	an_ethaddrspec_t	*eth = data;
	struct ip	*ip = (struct ip *)(data + ETHIP_IPOFF);

	assert(psize >= ETHIP_PLOFF);
	assert(eth->type == htons(ETHERTYPE_IP));
	assert(moff == ETHIP_IPDSTOFF+4);

	assert(streamp == &StreamData[0] || streamp == &StreamData[1]);
	if (streamp == &StreamData[0]) {
		inif = INIF;
		outif = OUTIF;
	} else {
		inif = OUTIF;
		outif = INIF;
	}

	if (verbose)
		printf("Got packet from %s (%lx->%lx): %d bytes\n",
		       inif == INIF ? "INIF" : "OUTIF",
		       ntohl(ip->ip_src.s_addr), ntohl(ip->ip_dst.s_addr),
		       psize);

	if (dying)
		goto done;

	/*
	 * Validate the packet and decrement the TTL before going any further
	 */
	if (ipcksum(ip, sizeof(struct ip)) != 0 || --ip->ip_ttl <= 0) {
		if (verbose)
			printf("Forwarder: dropping packet from %s "
			       "(bad packet)\n",
			       inif == INIF ? "INIF" : "OUTIF");
		goto done;
	}

	/*
	 * Determine the ethaddr of the destination host.
	 * If not in our cache, we must ARP for it.
	 */
	if (arp_cache_lookup(outif, &ip->ip_dst, eth->daddr) != 0) {
		err = arp_arpfor(outif, &ip->ip_dst, eth->daddr);
		if (err) {
			if (verbose)
				printf("Forwarder: dropping packet from %s "
				       "(dest ARP failed)\n",
				       inif == INIF ? "INIF" : "OUTIF");
			goto done;
		}
		arp_cache_add(outif, &ip->ip_dst, eth->daddr);
	}

	/*
	 * Incrementally update the checksum to reflect change in TTL.
	 */
	{
		unsigned long cs = ip->ip_sum;

		cs += htons(0x0100);
		cs += (cs >> 16);
		ip->ip_sum = cs;
	}

	/*
	 * Update the ethernet header.
	 * Note: daddr filled in by call to arp_cache_lookup.
	 */
	memcpy(eth->saddr, outif == OUTIF ? &outifaddr : &inifaddr, 6);

	/*
	 * Forward the packet
	 */
	err = an_outchan_send(streamp->outchan, pbuf);
	if (err != 0) {
		if (verbose)
			printf("Forwarder: dropping packet from %s "
			       "(send failed)\n",
			       inif == INIF ? "INIF" : "OUTIF");
	}

 done:
	an_inchan_addpbuf(streamp->inchan, pbuf);
}

static unsigned int
computeCPUcps(int frac)
{
	unsigned int ans;

	/*
	 * cpuinfo.cps is the total cycles per second available on the machine.
	 */
	assert(cpuinfo.cps > 0);

	ans = cpuinfo.cps / frac;
	return ans;
}

/*
 * Setup a data stream to route IP packets from the
 * input interface to the output interface and visa versa.
 */
static void
data_stream_initflow(void *arg)
{
	struct stream	*streamp = (struct stream *) arg;
	an_mempool_t	*cpool   = an_mempool_current();
	an_flow_t	*curflow = an_flow_current();
	int		i, flags;
	an_error_t	err;
	void		*mem;
	unsigned short	val_s;
	unsigned long	val_l, mask_l;
	struct {
		an_chanaddr_t	addr;
		an_chanproto_t	proto;
		an_cpuspec_t	cpu;
		an_netspec_t	net;
		an_pbuf_t	*bufs[DATASTRM_MAXBUFS];
	} attr;
	
	/*
	 * Setup demux to match all IP packets destined for the outgoing
	 * interface's network.
	 */
	streamp->ipdemux = an_demuxkey_create(&streamp->ipdemux_data, curflow);
	if (streamp->ipdemux == NULL)
		FATAL_AN_ERROR(-1);
	flags = AN_DEMUXKEY_ETH_ANYSADDR|AN_DEMUXKEY_ETH_ANYDADDR;
	val_s = htons(ETHERTYPE_IP);
	err = an_demuxkey_addeth(streamp->ipdemux, 0, flags, 0, 0, val_s);
	if (err)
		FATAL_AN_ERROR(err);
	flags = AN_DEMUXKEY_IPV4_ANYSADDR|AN_DEMUXKEY_IPV4_ANYPROTO;
	val_l = (outif_ip.s_addr & outif_mask.s_addr);
	mask_l = outif_mask.s_addr;
	err = an_demuxkey_addipv4(streamp->ipdemux, 0, flags,
				  0, 0, val_l, mask_l, 0);
	if (err)
		FATAL_AN_ERROR(err);

	/*
	 * Input side #1, read from INIF
	 */
	memset(&attr, 0, sizeof attr);
	strcpy(attr.proto.data, aninfo[INIF].pname);
	attr.proto.len = strlen(attr.proto.data);
	attr.cpu.ttype = AN_TH_IMPLICIT;
	attr.cpu.threads.im.maxthreads = DATASTRM_MAXTHREADS;

	/* Allocate buffers for data forwarding stream */
	for (i = 0; i < DATASTRM_MAXBUFS; i++) {
		mem = an_mempool_alloc(cpool, sizeof(an_pbuf_t));
		assert(mem);
		attr.bufs[i] = an_pbuf_create(mem, AN_PBUF_ETH, 0, 0);
		if (attr.bufs[i] == NULL)
			FATAL_AN_ERROR(-1);
	}
	
	streamp->inchan = an_inchan_create(&streamp->inchan_data,
					   curflow, streamp->ipdemux,
					   &attr.proto, &attr.addr,
					   attr.bufs, DATASTRM_MAXBUFS,
					   &attr.cpu, ee_ipData, streamp);
	if (streamp->inchan == 0)
		FATAL_AN_ERROR(-1);
	
	/*
	 * Output side #1, output to OUTIF
	 */
	memset(&attr, 0, sizeof attr);
	strcpy(attr.proto.data, aninfo[OUTIF].pname);
	attr.proto.len = strlen(attr.proto.data);
	attr.net.bandwidth = DATASTRM_DOWN_BANDW;

	streamp->outchan = an_outchan_create(&streamp->outchan_data,
					    curflow,
					    &attr.proto, &attr.addr,
					    &attr.net);
	if (streamp->outchan == 0)
		FATAL_AN_ERROR(-1);
	assert(streamp->outchan == &streamp->outchan_data);
	printf(__FUNCTION__ ": DATA downstream channels ready: "
	       "In:%p->Out:%p (bandwidth=%d)\n",
	       streamp->inchan, streamp->outchan, attr.net.bandwidth);

	/*
	 * Setup the reverse direction.
	 * Note that streamp points to the first of two contiguous
	 * Stream data structs, so we just advance the pointer to get to
	 * the second.
	 */
	streamp++;

	/*
	 * Setup demux to match all IP packets destined for the incoming
	 * interface's network.
	 */
	streamp->ipdemux = an_demuxkey_create(&streamp->ipdemux_data, curflow);
	if (streamp->ipdemux == NULL)
		FATAL_AN_ERROR(-1);
	flags = AN_DEMUXKEY_ETH_ANYSADDR|AN_DEMUXKEY_ETH_ANYDADDR;
	val_s = htons(ETHERTYPE_IP);
	err = an_demuxkey_addeth(streamp->ipdemux, 0, flags, 0, 0, val_s);
	if (err)
		FATAL_AN_ERROR(err);
	flags = AN_DEMUXKEY_IPV4_ANYSADDR|AN_DEMUXKEY_IPV4_ANYPROTO;
	val_l = (inif_ip.s_addr & inif_mask.s_addr);
	mask_l = inif_mask.s_addr;
	err = an_demuxkey_addipv4(streamp->ipdemux, 0, flags,
				  0, 0, val_l, mask_l, 0);
	if (err)
		FATAL_AN_ERROR(err);

	/*
	 * Input side #2, read from OUTIF
	 */
	memset(&attr, 0, sizeof attr);
	strcpy(attr.proto.data, aninfo[OUTIF].pname);
	attr.proto.len = strlen(attr.proto.data);
	attr.cpu.ttype = AN_TH_IMPLICIT;
	attr.cpu.threads.im.maxthreads = DATASTRM_MAXTHREADS;

	/* Allocate buffers for data forwarding stream */
	for (i = 0; i < DATASTRM_MAXBUFS; i++) {
		mem = an_mempool_alloc(cpool, sizeof(an_pbuf_t));
		assert(mem);
		attr.bufs[i] = an_pbuf_create(mem, AN_PBUF_ETH, 0, 0);
		if (attr.bufs[i] == NULL)
			FATAL_AN_ERROR(-1);
	}
	
	streamp->inchan = an_inchan_create(&streamp->inchan_data,
					   curflow, streamp->ipdemux,
					   &attr.proto, &attr.addr,
					   attr.bufs, DATASTRM_MAXBUFS,
					   &attr.cpu, ee_ipData, streamp);
	if (streamp->inchan == 0)
		FATAL_AN_ERROR(-1);
	
	/*
	 * Output side #2, output to INIF
	 */
	memset(&attr, 0, sizeof attr);
	strcpy(attr.proto.data, aninfo[INIF].pname);
	attr.proto.len = strlen(attr.proto.data);
	attr.net.bandwidth = DATASTRM_UP_BANDW;

	streamp->outchan = an_outchan_create(&streamp->outchan_data,
					    curflow,
					    &attr.proto, &attr.addr,
					    &attr.net);
	if (streamp->outchan == 0)
		FATAL_AN_ERROR(-1);
	assert(streamp->outchan == &streamp->outchan_data);
	printf(__FUNCTION__ ": DATA upstream channels ready: "
	       "In:%p->Out:%p (bandwidth=%d)\n",
	       streamp->inchan, streamp->outchan, attr.net.bandwidth);
}

static void
setup_data_stream(void)
{
	an_cpuspec_t	cs;
	an_memspec_t	ms;

	memset(&cs, 0, sizeof cs);
	memset(&ms, 0, sizeof ms);

	ms.nchunks = (32*1024) / NODEOS_MEMPOOL_CHUNKSIZE;
	ms.pool = an_mempool_current();
	
	/*
	 * Note, these are setting for the *flow* not the inchans,
	 * which is where the relevant DATASTRM limits are used.
	 */
	cs.ttype = AN_TH_IMPLICIT;
	cs.threads.im.maxthreads = DATASTRM_MAXTHREADS;
	cs.threads.im.stacksize  = DATASTRM_THREAD_STACKSIZE;
	cs.rate = computeCPUcps(DATASTRM_CPUFRAC_FLOW);

	printf(__FUNCTION__ ": Creating Data flow (threads=%d, cpurate=%d)\n",
	       cs.threads.im.maxthreads, cs.rate);
	StreamData[0].flow = an_flow_create(&StreamData[0].flow_data,
					    an_cred_current(), &cs, &ms,
					    data_stream_initflow,
					    &StreamData[0], 0);
	if (StreamData[0].flow == 0)
		FATAL_AN_ERROR(-1);
}

/*
 * Control Stream
 */
static void
ee_udpipCtl(an_pbuf_t *pbuf, void *arg, int ignored)
{
	void		*data = an_pbuf_data(pbuf);
	struct {
		struct ip	ip;
		struct udphdr	udp;
		ctrl_packet_t   packet;
	} *udpip = data;
	ctrl_packet_t	*ctrlp = &udpip->packet;
	int i;

	if (dying)
		goto done;

	/* byte-order-fixup the control packet */
	ctrlp->opcode = ntohl(ctrlp->opcode);
	ctrlp->subopcode = ntohl(ctrlp->subopcode);

	if (verbose)
		printf("Control Packet: opcode:%d, subop:%d\n",
		       ctrlp->opcode, ctrlp->subopcode);

	switch (ctrlp->opcode) {
	default:
		printf("Control Packet: Unknown opcode %d\n", ctrlp->opcode);
		goto done;

	case CTRLPKTOP_FOO:
		/* The debug op */
		printf("Test packet: subopcode=%d; ", ctrlp->subopcode);
		for (i = 0; i < 4; i++)
			printf("data[%d]=%#x ", i, ctrlp->data[i]);
		printf("\n");
		goto done;

	case CTRLPKTOP_NETHOG:
		an_mutex_lock(&nethog_mux);
		switch (ctrlp->subopcode) {
		default:
			printf("NetHog Control: "
			       "Unknown subopcode %d\n", ctrlp->subopcode);
			goto done;
		case NETHOGCTRL_ON:
			printf("NetHog Control: on\n");
			nethog_on = 1;
			an_cond_signal(&nethog_cond);
			break;
		case NETHOGCTRL_OFF:
			printf("NetHog Control: off\n");
			nethog_on = 0;
			break;
		case NETHOGCTRL_BANDW:
			/* XXX no interface for setting bw yet. */
			printf("NetHog control: ignored bw change: %d\n", ctrlp->data[0]);
			break;
		}
		an_mutex_unlock(&nethog_mux);
		break;
		
	case CTRLPKTOP_CPUHOG:
		switch (ctrlp->subopcode) {
		default:
			printf("CPUHog Control: "
			       "Unknown subopcode %d\n", ctrlp->subopcode);
			goto done;
		case CPUHOGCTRL_ON:
			printf("CPUHog Control: on\n");
			cpuhog_start_threads();
			break;
		case CPUHOGCTRL_OFF:
			printf("CPUHog Control: off\n");
			cpuhog_pause_threads();
			break;
		case CPUHOGCTRL_RATE:
			/* XXX no interface for setting rate yet. */
			printf("CPUHog control: ignored rate change: %d\n", ctrlp->data[0]);
			break;
		}
		break;

	case CTRLPKTOP_PQ:
		switch (ctrlp->subopcode) {
		default:
			printf("PQ Control: "
			       "Unknown subopcode %d\n", ctrlp->subopcode);
			goto done;
		case PQCTRL_ON:
			printf("PQ Control: on\n");
			pq_enable();
			break;
		case PQCTRL_OFF:
			printf("PQ Control: off\n");
			pq_disable();
			break;
		}
		break;
	case CTRLPKTOP_CPUSCHED:
		switch (ctrlp->subopcode) {
		default:
			printf("CPUSched Control: "
			       "Unknown subopcode %d\n", ctrlp->subopcode);
			goto done;
		case CPUSCHEDCTRL_ON:
			printf("CPUSched Control: on\n");
			stride_scheduler_enable();
			break;
		case CPUSCHEDCTRL_OFF:
			printf("CPUSched Control: off\n");
			stride_scheduler_disable();
			break;
		}
		break;
	case CTRLPKTOP_EXIT:
		printf("Exit.\n");
		stat_send((void *)0x42, StreamStat.outchan);
		ee_die(NULL, NULL); /* XXX bogus args */
		goto done;
	case CTRLPKTOP_VERBOSE:
		switch (ctrlp->subopcode) {
		default:
			printf("Verbose Control: "
			       "Unknown subopcode %d\n", ctrlp->subopcode);
			goto done;
		case VERBOSECTRL_ON:
			printf("Verbose Control: on\n");
			verbose = 1;
			break;
		case VERBOSECTRL_OFF:
			printf("Verbose Control: off\n");
			verbose = 0;
			break;
		}
		goto done;
	case CTRLPKTOP_STATS:
		switch (ctrlp->subopcode) {
		default:
			printf("Stats Control: "
			       "Unknown subopcode %d\n", ctrlp->subopcode);
			goto done;
		case STATSCTRL_TOTAL:
			stat_print(1);
			break;
		case STATSCTRL_DELTA:
			stat_print(0);
			break;
		}
		goto done;
	}
 done:
	an_inchan_addpbuf(StreamCtl.inchan, pbuf);
}

static void
control_stream_initflow(void *arg)
{
	struct stream	*streamp = (struct stream *) arg;
	an_mempool_t	*cpool   = an_mempool_current();
	an_flow_t	*curflow = an_flow_current();
	int		i, flags;
	an_error_t	err;
	unsigned char	val_c;
	unsigned short	val_s;
	struct {
		an_chanaddr_t	addr;
		an_chanproto_t	proto;
		an_cpuspec_t	cpu;
		an_netspec_t	net;
		/* XXX this is a lot of stuff to put on the stack */
		an_pbuf_t*	bufs[CTRLSTRM_MAXBUFS];
	} attr;
	
	memset(&attr, 0, sizeof attr);


	/*
	 * Allocate input buffers.
	 */
	for (i = 0; i < CTRLSTRM_MAXBUFS; i++) {
		void *mem;
		mem = an_mempool_alloc(cpool, sizeof(an_pbuf_t));
		assert(mem); /* XXX */

		attr.bufs[i] = an_pbuf_create(mem, AN_PBUF_ETH, 0, 0);
		if (attr.bufs[i] == NULL)
			FATAL_AN_ERROR(-1);
		assert(attr.bufs[i] == mem);
	}

	/*
	 * Just an input channel matching UDP/IP traffic to the control port.
	 */
	streamp->ipdemux = an_demuxkey_create(&streamp->ipdemux_data,
					       curflow);
	assert(streamp->ipdemux != 0);

	flags = AN_DEMUXKEY_IPV4_ANYSADDR|AN_DEMUXKEY_IPV4_ANYDADDR;
	val_c = IPPROTO_UDP;
	err = an_demuxkey_addipv4(streamp->ipdemux, 0, flags,
				  0, 0, 0, 0, val_c);
	assert(err == 0);
	flags = AN_DEMUXKEY_UDP_ANYSPORT;
	val_s = htons(CTRLPORT);
	err = an_demuxkey_addudp(streamp->ipdemux, 0, flags, 0, val_s);
	assert(err == 0);

	sprintf(attr.proto.data, "if%d", aninfo[CTRLIF].id);
	attr.proto.len = strlen(attr.proto.data);
	attr.addr.eth.type = htons(ETHERTYPE_IP);
	/* attr.addr.eth.saddr already zero. */
	memcpy(attr.addr.eth.daddr, ctrlifaddr.daddr, 6);
	attr.cpu.ttype = AN_TH_IMPLICIT;
	attr.cpu.threads.im.maxthreads = CTRLSTRM_MAXTHREADS;

	streamp->inchan = an_inchan_create(&streamp->inchan_data,
					   curflow, streamp->ipdemux,
					   &attr.proto, &attr.addr, attr.bufs,
					   CTRLSTRM_MAXBUFS,
					   &attr.cpu, ee_udpipCtl, 0);
	if (streamp->inchan == NULL)
		FATAL_AN_ERROR(-1);
	assert(streamp->inchan == &streamp->inchan_data);

	printf(__FUNCTION__ ": CTRLSTRM channel ready: %p\n",
	       streamp->inchan);
}

static void
setup_control_stream(void)
{
	an_cpuspec_t	cs;
	an_memspec_t	ms;

	memset(&cs, 0, sizeof cs);
	memset(&ms, 0, sizeof ms);

	ms.nchunks = (32*1024) / NODEOS_MEMPOOL_CHUNKSIZE;
	ms.pool = an_mempool_current();
	
	/*
	 * Note, this limit is for the *flow* not the inchan, which
	 * is where the relevant rate limits are imposed.
	 */
	cs.ttype = AN_TH_IMPLICIT;
	cs.threads.im.maxthreads = 1;
	cs.threads.im.stacksize  = CTRLSTRM_THREAD_STACKSIZE;
	cs.rate = computeCPUcps(CTRLSTRM_CPUFRAC_FLOW);

	printf(__FUNCTION__ ": Creating Control flow (threads=%d, cpurate=%d)\n",
	       cs.threads.im.maxthreads, cs.rate);
	StreamCtl.flow = an_flow_create(&StreamCtl.flow_data,
					an_cred_current(), &cs, &ms,
					control_stream_initflow,
					&StreamCtl, 0);
	if (StreamCtl.flow == 0)
		FATAL_AN_ERROR(-1);
	assert(StreamCtl.flow == &StreamCtl.flow_data);
}

static void
stat_send(void *event, void *arg)
{
	an_chan_t *outchan = (an_chan_t*)arg;
	an_error_t err;
	static int tick;
#ifndef PRINTSTATS
	an_pbuf_t *pbuf;
	struct udpip {
		struct ip	ip;
		struct udphdr	udp;
	} *udpip;
	void *udpPayload;
#endif

	if (!running || dying)
		goto skip;

	tick++;

#ifdef PRINTSTATS
	printf("Stat record #%d:\n", tick);
	stat_print(0);
#else
	//	printf("sending stat.\n");

	/* Get a pbuf */
	pbuf = get_stat_pbuf();
	udpip = (struct udpip*)an_pbuf_data(pbuf);
	udpPayload = (udpip+1);

	/* Paste latest stats into it. */
	{
		stat_packet_t *stats;

		err = an_pbuf_setsize(pbuf, sizeof *udpip + sizeof *stats, 0);
		assert(err == 0);

		stats = (stat_packet_t *)udpPayload;

		err = an_flow_stats(StreamData[0].flow, &stats->dataflow);
		assert(err == 0);
		err = an_flow_stats(StreamNetHog.flow, &stats->nethogflow);
		assert(err == 0);
		err = an_flow_stats(StreamCPUHog.flow, &stats->cpuhogflow);
		assert(err == 0);

		err = an_chan_stats(StreamData[0].outchan, &stats->datachan);
		assert(err == 0);
#if 0
		err = an_chan_stats(StreamData[1].outchan, &stats->datachan2);
		assert(err == 0);
#endif
		err = an_chan_stats(StreamNetHog.outchan, &stats->nethogchan);
		assert(err == 0);

		/* we are all one big happy mempool */
		err = an_mempool_stats(an_mempool_current(), &stats->mempool);
		assert(err == 0);

		stats->version = htonl(STATS_VERSION_MOAB);

		if ((unsigned)event == 0x42) {
			/* XXX hack to signal demo exit */
			stats->timestamp.tv_sec = -1;
			stats->timestamp.tv_nsec = -1;
		} else {
			an_gettime(&stats->timestamp);
			stats->timestamp.tv_sec =
				htonl(stats->timestamp.tv_sec);
			stats->timestamp.tv_nsec =
				htonl(stats->timestamp.tv_nsec);
		}
	}

	/* Fixup the IP header for the pbuf */
	udpip->ip.ip_len = htons(an_pbuf_size(pbuf));
	udpip->ip.ip_id = ip_magic_id++; /* XXX */
	udpip->ip.ip_sum = 0;
	udpip->ip.ip_sum = ipcksum(&udpip->ip, sizeof(struct ip));
	udpip->udp.uh_ulen = htons(an_pbuf_size(pbuf) - sizeof(struct ip));
	udpip->udp.uh_sum = 0; /* Zero means no chksum computed. */

	an_outchan_send(outchan, pbuf);

	/* Recycle the pbuf */
	put_stat_pbuf(pbuf);
#endif

 skip:
	/*
	 * Install another event
	 */
	err = an_event_schedule(stat_send, (void*)outchan, STATFREQ * 1000);
	if (err != 0)
		FATAL_AN_ERROR(err);
}

static void
stat_stream_initflow(void *arg)
{
	struct stream	*streamp = (struct stream *) arg;
	an_flow_t	*curflow = an_flow_current();
	an_error_t	err;
	struct {
		an_chanaddr_t	addr;
		an_chanproto_t	proto;
		an_netspec_t	net;
	} attr;

	/*
	 * Allocate output buffers.
	 */
	setup_stat_buffers();

	/*
	 * Just an output channel.  To avoid dependencies on non-ETH
	 * channels, we do all the IP/UDP marshalling ourselves...
	 *
	 * See init_stat_buffer() for details on the packet headers.
	 */
	memset(&attr, 0, sizeof attr);
	sprintf(attr.proto.data, "if%d", aninfo[STATIF].id);
	attr.proto.len = strlen(attr.proto.data);
	attr.addr.eth.type = htons(ETHERTYPE_IP);
	/* attr.addr.eth.saddr already zero */
	memcpy(attr.addr.eth.daddr, statdstaddr.daddr, 6);
	attr.net.bandwidth = STATSTRM_BANDW;

	streamp->outchan = an_outchan_create(&streamp->outchan_data,
					     curflow, 
					     &attr.proto, &attr.addr,
					     &attr.net);

	if (streamp->outchan == NULL)
		FATAL_AN_ERROR(-1);
	assert(streamp->outchan == &streamp->outchan_data);

	/*
	 * Setup a timer to serve up a piping hot plate of stats.
	 */
	err = an_event_schedule(stat_send, (void*)streamp->outchan,
				STATFREQ * 1000);
	if (err != 0)
		FATAL_AN_ERROR(err);

	printf(__FUNCTION__ ": STATSTRM channel ready: %p (bandwidth=%d)\n",
	       streamp->outchan, attr.net.bandwidth);
}

static void
setup_stat_stream(void)
{
	an_cpuspec_t	cs;
	an_memspec_t	ms;

	memset(&cs, 0, sizeof cs);
	memset(&ms, 0, sizeof ms);

	ms.nchunks = (32*1024) / NODEOS_MEMPOOL_CHUNKSIZE;
	ms.pool = an_mempool_current();
	
	/*
	 * Note, this limit is for the *flow* not the inchan, which
	 * is where the relevant rate limits are imposed.
	 */
	cs.ttype = AN_TH_IMPLICIT;
	cs.threads.im.maxthreads = 1;
	cs.threads.im.stacksize  = STATSTRM_THREAD_STACKSIZE;
	cs.rate = computeCPUcps(STATSTRM_CPUFRAC_FLOW);

	printf(__FUNCTION__ ": Creating Stat flow (threads=%d, cpurate=%d)\n",
	       cs.threads.im.maxthreads, cs.rate);
	StreamStat.flow = an_flow_create(&StreamStat.flow_data,
					 an_cred_current(), &cs, &ms,
					 stat_stream_initflow,
					 &StreamStat, 0);
	if (StreamStat.flow == 0)
		FATAL_AN_ERROR(-1);
	assert(StreamStat.flow == &StreamStat.flow_data);
}

static void
nethog_stream_initflow(void *arg)
{
	struct stream	*streamp = (struct stream *) arg;
	an_flow_t	*curflow = an_flow_current();
	struct {
		an_chanaddr_t	addr;
		an_chanproto_t	proto;
		an_netspec_t	net;
	} attr;
	
	if (an_mutex_create(&nethog_mux) == 0)
		FATAL_AN_ERROR(AN_ERR_NOMEMORY);
	if (an_cond_create(&nethog_cond) == 0)
		FATAL_AN_ERROR(AN_ERR_NOMEMORY);

	/*
	 * Allocate output buffers.
	 */
	setup_nethog_buffers();

	/*
	 * Just an output stream. A raw ethernet channel.
	 */
	memset(&attr, 0, sizeof attr);
	strncpy(attr.proto.data,
		aninfo[OUTIF].pname, sizeof(aninfo[OUTIF].pname));
	attr.proto.len = strlen(attr.proto.data);
	/* attr.addr.eth.type already zero */
	/* attr.addr.eth.saddr already zero */
	/* attr.addr.eth.daddr already zero */
	attr.net.bandwidth = NETHOGSTRM_BANDW;

	streamp->outchan = an_outchan_create(&streamp->outchan_data,
					    curflow,
					    &attr.proto, &attr.addr,
					    &attr.net);
	
	assert(streamp->outchan != 0);

	printf(__FUNCTION__ ": NETHOGSTRM channel ready: %p (bandwidth=%d)\n",
	       streamp->outchan, attr.net.bandwidth);

	/*
	 * Fire off packets from this thread.
	 */
	while (!dying) {
		an_mutex_lock(&nethog_mux);
		if (nethog_on) {
			an_pbuf_t *pbuf;

			an_mutex_unlock(&nethog_mux);

			/*
			 * Send a pbuf.
			 * If it fails, oh well.
			 */
			pbuf = get_nethog_pbuf();
			an_outchan_send(streamp->outchan, pbuf);
			put_nethog_pbuf(pbuf);
#ifdef NETHOGSTRM_DEBUG
			{
				static int ct = 0;
				if (ct++ == (50 * 1000))
				{
					fprintf(stderr, "X ");
					ct = 0;
				}
			}
#endif
		} else {
			/* NetHog is NOT on.  Wait for it */
			an_cond_wait(&nethog_cond, &nethog_mux);
			an_mutex_unlock(&nethog_mux);
		}
	}
}


static void
setup_nethog_stream(void)
{
	an_cpuspec_t	cs;
	an_memspec_t	ms;

	memset(&cs, 0, sizeof cs);
	memset(&ms, 0, sizeof ms);

	ms.nchunks = (32*1024) / NODEOS_MEMPOOL_CHUNKSIZE;
	ms.pool = an_mempool_current();

	cs.ttype = AN_TH_IMPLICIT;
	cs.threads.im.maxthreads = NETHOGSTRM_MAXTHREADS;
	cs.threads.im.stacksize  = NETHOGSTRM_THREAD_STACKSIZE;
	cs.rate = computeCPUcps(NETHOGSTRM_CPUFRAC_FLOW);

	printf(__FUNCTION__ ": Creating NetHog flow (threads=%d, cpurate=%d)\n",
	       cs.threads.im.maxthreads, cs.rate);
	StreamNetHog.flow = an_flow_create(&StreamNetHog.flow_data,
					an_cred_current(), &cs, &ms,
					nethog_stream_initflow,
					&StreamNetHog, 0);
	assert(StreamNetHog.flow != 0);
}

static an_mutex_t	cpuhog_mux;
static an_cond_t	cpuhog_cond;

static void
cpuhog_pause_threads(void)
{
	cpuhog_on = 0;
}

static void
cpuhog_start_threads(void)
{
	an_mutex_lock(&cpuhog_mux);
	cpuhog_on = 1;
	an_cond_broadcast(&cpuhog_cond);
	an_mutex_unlock(&cpuhog_mux);
}

static void
cpuhog_thread_wait(void)
{
	an_mutex_lock(&cpuhog_mux);
	an_cond_wait(&cpuhog_cond, &cpuhog_mux);
	an_mutex_unlock(&cpuhog_mux);
}

static void
cpuhog_cycle_waster(void *arg)
{
#ifdef CPUHOGSTRM_DEBUG
	int i = 0;
	int id = (int)arg;
#endif

	while(1)
	{
		if (dying)
			return;

		if (!cpuhog_on)
			cpuhog_thread_wait();

#ifdef CPUHOGSTRM_DEBUG
		i++;
		if (i == (1 * 1000 * 1000))
		{
			fprintf(stderr, "%02d ", id);

			/* last one started is usually last to print... */
			if (id == (CPUHOGSTRM_MAXTHREADS-2))
				fprintf(stderr, "\n");
			i = 0;
		}
#endif
	}
}

static void
cpuhog_stream_initflow(void *arg)
{
	/*struct stream	*streamp = (struct stream *) arg;*/
	an_flow_t *curflow = an_flow_current();
	an_error_t rc;
	int i;
	
	if (an_mutex_create(&cpuhog_mux) == 0)
		FATAL_AN_ERROR(AN_ERR_NOMEMORY);
	if (an_cond_create(&cpuhog_cond) == 0)
		FATAL_AN_ERROR(AN_ERR_NOMEMORY);

	/*
	 * Create & fire off n-1 threads to waste time, then do the same.
	 */
	for (i = 0; i < CPUHOGSTRM_MAXTHREADS-1; i++) {
		rc = an_flow_startthread(curflow,
					 cpuhog_cycle_waster, (void *)i, 0);
		assert(rc == 0);
	}
	
	printf(__FUNCTION__ ": CPUHOG threads ready.\n");

	cpuhog_cycle_waster((void*)(CPUHOGSTRM_MAXTHREADS-1));
}

static void
setup_cpuhog_stream(void)
{
	an_cpuspec_t	cs;
	an_memspec_t	ms;

	memset(&cs, 0, sizeof cs);
	memset(&ms, 0, sizeof ms);

	ms.nchunks = (32*1024) / NODEOS_MEMPOOL_CHUNKSIZE;
	ms.pool = an_mempool_current();

	/*
	 * Note CPUHOGSTRM_MAXTHREADS-1 threads are created explicitly
	 * in cpuhog_stream_initflow().
	 */
	cs.ttype = AN_TH_IMPLICIT;
	cs.threads.im.maxthreads = CPUHOGSTRM_MAXTHREADS;
	cs.threads.im.stacksize  = CPUHOGSTRM_THREAD_STACKSIZE;
	cs.rate = computeCPUcps(CPUHOGSTRM_CPUFRAC_FLOW);

	printf(__FUNCTION__ ": Creating CPUHog flow (threads=%d, cpurate=%d)\n",
	       cs.threads.im.maxthreads, cs.rate);
	StreamCPUHog.flow = an_flow_create(&StreamCPUHog.flow_data,
					   an_cred_current(), &cs, &ms,
					   cpuhog_stream_initflow,
					   &StreamCPUHog, 0);
	if (StreamCPUHog.flow == 0)
		FATAL_AN_ERROR(-1);
	assert(StreamCPUHog.flow == &StreamCPUHog.flow_data);
}

static void
ee_die(void *event, void *arg)
{
	/* Signal all the flows to stop what they're doing. */
	dying = 1;

	/* XXX kick the cpuhog threads? */

	assert(StreamData[0].outchan);
	assert(StreamData[1].outchan);
	assert(StreamNetHog.outchan);

	stat_print(1);

	teardown_nethog_buffers();
	teardown_stat_buffers();

	an_inchan_destroy(StreamData[0].inchan);
	an_outchan_destroy(StreamData[0].outchan);
	an_inchan_destroy(StreamData[1].inchan);
	an_outchan_destroy(StreamData[1].outchan);
	an_outchan_destroy(StreamNetHog.outchan);
	an_inchan_destroy(StreamCtl.inchan);
	
	/*
	 * This function is invoked in the context of the StreamCtl.flow.
	 * When the parent flow, 'initflow', is destroyed, that function
	 * won't return.
	 */

	an_flow_destroy(StreamData[0].flow);
	an_flow_destroy(StreamNetHog.flow);
	an_flow_destroy(StreamCPUHog.flow);
	arp_exit();

	/* Directly shut down moab.  Not the cleanest, but its easy. */
	an_moab_shutdown();
#if 0
	/*
	 * XXX FOLLOWING DOESN'T WORK as ee_die() can be invoked in
	 * either initflow or StreamCtl.flow.  (meaning one wouldn't
	 * be destroyed).
	 */
	an_flow_destroy(StreamCtl.flow);
	an_flow_destroy(initflow); /* XXX this won't return */
#endif
}

/*
 * Stat packet buffer cache
 */
static pbcache_t* stat_cache;

static void
init_stat_buffer(an_pbuf_t *pbuf)
{
	struct udpip {
		struct ip	ip;
		struct udphdr	udp;
	} *udpip;

	/* Minimum size, user must add more for udp payload size */
	an_pbuf_setsize(pbuf, sizeof(struct udpip), NULL);

	/*
	 * Setup packet to be delivered on a configured 'ifX'
	 * interface.  (Ether header is supplied for us, we need to
	 * add IP and UDP header.
	 *
	 * Fills in all fields except ip.ip_len, ip.ip_sum,
	 * udp.uh_ulen, and udp.uh_sum.
	 */
	udpip = (struct udpip*)(an_pbuf_data(pbuf));

	udpip->ip.ip_v = 4;
	udpip->ip.ip_hl = 20 >> 2; /* 20 bytes, cvted to word size.  No IP options. */
	udpip->ip.ip_tos = 0;
	udpip->ip.ip_len = 0; /* *** Filled in when packet is sent */
	udpip->ip.ip_id = 0;
	udpip->ip.ip_off = 0;
	udpip->ip.ip_ttl = 16; /* XXX */
	udpip->ip.ip_p = 17; /* UDP protocol */
	udpip->ip.ip_src = statif_ip;
	udpip->ip.ip_dst = statdst_ip;
	udpip->ip.ip_sum = 0; /* *** Can't fill until ip_len is set. */
	
	udpip->udp.uh_sport = htons(STATIFPORT);
	udpip->udp.uh_dport = htons(STATDSTPORT);
	udpip->udp.uh_ulen = 0; /* *** Filled in when packet is sent */
	udpip->udp.uh_sum = 0; /* *** Filled in when packet is sent */
}

static void
setup_stat_buffers()
{
	stat_cache = pbcache_create(STATSTRM_MAXBUFS, init_stat_buffer);
}

static void
teardown_stat_buffers(void)
{
	int n;

	pbcache_dump(stat_cache);
	n = pbcache_destroy(stat_cache, 0);
	if (n != 0)
		printf("* * * %d buffers not destroyed in stat_cache.\n", n);
}

static an_pbuf_t *
get_stat_pbuf()
{
	return pbcache_alloc(stat_cache, 1);
}

static void
put_stat_pbuf(an_pbuf_t *pbuf)
{
	pbcache_free(stat_cache, pbuf);
}


/*
 * NetHog packet buffer cache
 */

static pbcache_t *nethog_cache;
char bogus_ethaddr[6] = { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 }; 
static int allocfails, allocsleeps, allocspins;

static void
init_nethog_buffer(an_pbuf_t *pbuf)
{
	an_ethaddrspec_t *hdr;

	an_pbuf_setsize(pbuf, NETHOGSTRM_PKTSIZE, NULL);
	hdr = an_pbuf_data(pbuf);

	memcpy(hdr->daddr, bogus_ethaddr, sizeof hdr->daddr);
	memcpy(hdr->saddr, outifaddr.daddr, sizeof hdr->saddr);
	hdr->type = htons(ETHERTYPE_IP);
}

static void
setup_nethog_buffers()
{
	nethog_cache = pbcache_create(NETHOGSTRM_MAXBUFS, init_nethog_buffer);
}

static void
teardown_nethog_buffers(void)
{
	int n;

	printf("Nethog pbuf cache:\n%d alloc failures, %d sleeps, %d spins\n",
	       allocfails, allocsleeps, allocspins);
	pbcache_dump(nethog_cache);
	n = pbcache_destroy(nethog_cache, 0);
	if (n != 0)
		printf("* * * %d buffers not destroyed in nethog_cache.\n", n);
}

/*
 * Allocate a pbuffer.
 * If the allocation fails we sleep for 10ms.  10ms is the likely scheduling
 * granularity and we should be able to keep the network pumping along for
 * that long if we have at least 80 queued packets (i.e. 80 pbufs in the
 * cache) since we can send ~8000 full-sized packets per second at 100Mb/sec.
 */
static an_pbuf_t *
get_nethog_pbuf()
{
	an_pbuf_t *pbuf;
	int fails = 0, sleeps = 0;

	while ((pbuf = pbcache_alloc(nethog_cache, 0)) == 0) {
		an_timespec_t ts;

		fails++;
		sleeps++;
		ts.tv_sec = 0;
		ts.tv_nsec = 10000000;
		an_thread_sleep(&ts);
	}
	if (fails) {
		allocfails++;
		allocspins += fails;
		allocsleeps += sleeps;
	}
	return pbuf;
}

static void
put_nethog_pbuf(an_pbuf_t *pbuf)
{
	pbcache_free(nethog_cache, pbuf);
}

static void
fatal_an_error(const char* file, const char *func, int line, an_error_t err)
{
	const char* msg;

	/* Save message before invoking another an_ func */
	msg = strdup(an_error_message());

	/* Shut everyone else up while I'm talking. */
	/* an_thread_setprio(an_thread_current(), AN_THREAD_PRIO_EXCLUSIVE); */

	if (err == -1)
	{
		printf("%s:%s():%d fatal AN error:\n\t%s\n",
		       file, func, line, an_error_message());
	}
	else
	{
		printf("%s:%s():%d fatal AN error: (%d)\n\t %s\n",
		       file, func, line, err, an_error_message());
	}

	exit(11);
}

static void
stat_print(int showtotal)
{
	an_error_t err;
	unsigned seconds;
	an_timespec_t curtime;
	static an_timespec_t lasttime;

	if (showtotal) {
		an_gettime(&curtime);
		if (curtime.tv_nsec < starttime.tv_nsec) {
			curtime.tv_nsec += 1000000;
			curtime.tv_sec--;
		}
		if (curtime.tv_nsec - starttime.tv_nsec > 500000)
			curtime.tv_sec++;
		seconds = curtime.tv_sec - starttime.tv_sec;

		printf("Stats after %d seconds:\n", seconds);

		/*
		 * Data stream stats
		 */
		err = moab_flow_printstats(StreamData[0].flow, 1,
					   "Data", seconds, 0);
		assert(err == 0);
		err = moab_chan_printstats(StreamData[0].inchan, 1,
					   " Data1", seconds, 0);
		assert(err == 0);
		err = moab_chan_printstats(StreamData[0].outchan, 1,
					   " Data1", seconds, 0);
		assert(err == 0);
		err = moab_chan_printstats(StreamData[1].inchan, 0,
					   " Data2", seconds, 0);
		assert(err == 0);
		err = moab_chan_printstats(StreamData[1].outchan, 0,
					   " Data2", seconds, 0);
		assert(err == 0);

		/*
		 * Hog stats
		 */
		err = moab_flow_printstats(StreamCPUHog.flow, 1,
					   "CPUHog", seconds, 0);
		assert(err == 0);
		err = moab_flow_printstats(StreamNetHog.flow, 1,
					   "NetHog", seconds, 0);
		assert(err == 0);
		err = moab_chan_printstats(StreamNetHog.outchan, 1,
					   " NetHog", seconds, 0);
		assert(err == 0);

		/*
		 * Mempool stats
		 */
		err = moab_mempool_printstats(an_mempool_current(), 1,
					      "Mempool", 0);
		assert(err == 0);
	} else {
		if (lasttime.tv_sec == 0)
			lasttime = starttime;

		an_gettime(&curtime);
		if (curtime.tv_nsec < lasttime.tv_nsec) {
			curtime.tv_nsec += 1000000;
			curtime.tv_sec--;
		}
		if (curtime.tv_nsec - lasttime.tv_nsec > 500000)
			curtime.tv_sec++;
		seconds = curtime.tv_sec - lasttime.tv_sec;
		lasttime = curtime;

		printf("Stats for last %d seconds:\n", seconds);

		/*
		 * Data stream stats
		 */
		err = moab_flow_printstatsdelta(StreamData[0].flow, 1,
						"Data", seconds,
						&StreamData[0].flowcookie);
		assert(err == 0);
		err = moab_chan_printstatsdelta(StreamData[0].inchan, 1,
						" Data1", seconds,
						&StreamData[0].incookie);
		assert(err == 0);
		err = moab_chan_printstatsdelta(StreamData[0].outchan, 1,
						" Data1", seconds,
						&StreamData[0].outcookie);
		assert(err == 0);
		err = moab_chan_printstatsdelta(StreamData[1].inchan, 0,
						" Data2", seconds,
						&StreamData[1].incookie);
		assert(err == 0);
		err = moab_chan_printstatsdelta(StreamData[1].outchan, 0,
						" Data2", seconds,
						&StreamData[1].outcookie);
		assert(err == 0);

		/*
		 * Hog stats
		 */
		err = moab_flow_printstatsdelta(StreamCPUHog.flow, 1,
						"CPUHog", seconds,
						&StreamCPUHog.flowcookie);
		assert(err == 0);
		err = moab_flow_printstatsdelta(StreamNetHog.flow, 1,
						"NetHog", seconds,
						&StreamNetHog.flowcookie);
		assert(err == 0);
		err = moab_chan_printstatsdelta(StreamNetHog.outchan, 1,
						" NetHog", seconds,
						&StreamNetHog.outcookie);
		assert(err == 0);

		/*
		 * Mempool stats
		 * XXX not a delta
		 */
		err = moab_mempool_printstats(an_mempool_current(), 1,
					      "Mempool", 0);
		assert(err == 0);
	}
	printf("\n");
}
