#
# Copyright (c) 1999, 2000, 2001 University of Utah and the Flux Group.
# All rights reserved.
# This file is part of Janos.
# 
# Janos is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License, version 2, as
# published by the Free Software Foundation.
# 
# Janos is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details, in the file
# COPYING.  If that file is not present, write to the Free
# Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
# 02111-1307, USA
# 
# The University of Utah and the Flux Group requests users to
# return to flux-dist@cs.utah.edu any improvements that they make.
#

# The basics
EES = 	null flowtest chantest eventtest socktest threadtest rootflow fs
EES += 	debugtest
# scheduling
EES += packsched cputest
ifneq ($(HOST_OS),solaris)
# require raw ethernet access
EES +=	pingreply forward packetdump ipforward
endif
ifneq ($(HOST_OS),oskit)
EES +=	anepdump
endif
ifneq (,$(findstring CHAN_CLICK,$(CHANNELS)))
EES += udptest cutpingreply
endif
# oskit only
# XXX requires tweaking oskit/pq or it won't even compile
#ifeq ($(HOST_OS),oskit)
#EES += demo
#endif
EES_P =	$(addsuffix _p, $(EES))

CHECKABLE_EES = null eventtest socktest threadtest rootflow

# mkmbimage additional command line arguments
#demo_MBFILES = "$(NODEOS_SRCDIR)/envs/mike-demo.config:demo.config"

ifeq ($(ENABLE_PROFILING),yes)
all: $(EES) all_p
else
all: $(EES)
endif

all_p: $(EES_P)

### Setup the architecture-specific variables

include $(NODEOS_SRCDIR)/arch.mk

### Define some global settings for all EEs

CPPFLAGS = $(OPT_CFLAGS) $(ARCH_CFLAGS) -I../ -I$(NODEOS_SRCDIR)

#CFLAGS_WARNINGS=-W -Wcast-align -Wwrite-strings -Wstrict-prototypes -Wmissing-prototypes -Wnested-externs -Winline
CFLAGS_WARNINGS = -Wwrite-strings -Wstrict-prototypes -Winline

ifneq ($(HOST_OS),solaris)	# these are too annoying on solaris
CFLAGS_WARNINGS += -Wcast-align -Wshadow
endif

CFLAGS += 	-Wall $(CPPFLAGS) $(CFLAGS_WARNINGS)

NODEOS_OBJDIR := ../src
NODEOS_LIBDIR := ../lib
ifneq (,$(findstring CHAN_CLICK,$(CHANNELS)))
NODEOS_LIBS := 		-u main -L$(NODEOS_LIBDIR) -lmoabutil \
			-L$(NODEOS_OBJDIR) -lnodeos \
			-L$(CLICK_INSTDIR)/lib -lclick
NODEOS_PLIBS := 	-u main -pg -realtime -L$(NODEOS_LIBDIR) -lmoabutil_p \
			-L$(NODEOS_OBJDIR) -lnodeos_p \
			-L$(CLICK_INSTDIR)/lib -lclick_p
else
NODEOS_LIBS := 		-u main -L$(NODEOS_LIBDIR) -lmoabutil \
			-L$(NODEOS_OBJDIR) -lnodeos
NODEOS_PLIBS := 	-u main -pg -realtime -L$(NODEOS_LIBDIR) -lmoabutil_p \
			-L$(NODEOS_OBJDIR) -lnodeos_p
endif
NODEOS_LIB_DEPS := 	$(NODEOS_OBJDIR)/libnodeos.a \
			$(NODEOS_LIBDIR)/libmoabutil.a
NODEOS_PLIB_DEPS := 	$(NODEOS_OBJDIR)/libnodeos_p.a \
			$(NODEOS_LIBDIR)/libmoabutil_p.a


### Define the EEs

EE=null
EESRCS=null.c
include $(SRCDIR)/define_env.mk

EE=flowtest
EESRCS=flowtest.c
include $(SRCDIR)/define_env.mk

EE=chantest
EESRCS=chantest.c
include $(SRCDIR)/define_env.mk

EE=pingreply
EESRCS=pingreply.c
include $(SRCDIR)/define_env.mk

EE=packetdump
EESRCS=packetdump.c
include $(SRCDIR)/define_env.mk

EE=socktest
EESRCS=socktest.c
include $(SRCDIR)/define_env.mk

EE=forward
EESRCS=forward.c
include $(SRCDIR)/define_env.mk

EE=ipforward
EESRCS=ipforward.c
include $(SRCDIR)/define_env.mk

EE=threadtest
EESRCS=threadtest.c
include $(SRCDIR)/define_env.mk

EE=eventtest
EESRCS=eventtest.c
include $(SRCDIR)/define_env.mk

EE=rootflow
EESRCS=rootflow.c
include $(SRCDIR)/define_env.mk

EE=cputest
EESRCS=cputest.c
include $(SRCDIR)/define_env.mk

EE=cutpingreply
EESRCS=cutpingreply.c
include $(SRCDIR)/define_env.mk

EE=udptest
EESRCS=udptest.c
include $(SRCDIR)/define_env.mk

EE=packsched
EESRCS=packsched.c
include $(SRCDIR)/define_env.mk

EE=anepdump
EESRCS=anepdump.c
include $(SRCDIR)/define_env.mk

EE=demo
EESRCS=demo.c
MBFILES = $(demo_MBFILES)
include $(SRCDIR)/define_env.mk

EE=debugtest
EESRCS=debugtest.c
include $(SRCDIR)/define_env.mk

EE=fs
EESRCS=fs.c
include $(SRCDIR)/define_env.mk

EE=apicalls
EESRCS=apicalls.c
include $(SRCDIR)/define_env.mk

clean:
	$(SILENT_MAKE_INFORM) "Cleaning out *.o *.po *.d and the EEs."
	rm -f core *.d *.core *.o *.po $(EES) $(EES_P) \
	      cutpingreply *.mb

depend:
	@echo "WARNING: depend target is deprecated."

distclean: clean
	$(SILENT_MAKE_INFORM) "Scrubbing EE directory."
	rm -f GNUmakefile

install install_p: all
	@echo The example EEs do not get installed.

ifeq ($(HOST_OS),oskit)
check:
	@echo "Cannot check EEs for an OSKit build.  (Must be able to run on build host.)"

else

check:
	@for test in $(filter $(CHECKABLE_EES), $(EES)); do \
		echo " "; \
		echo "---- Running test $$test"; \
		./$$test || echo "$$test failed."; \
		echo " "; \
	done \

endif

.PHONY: all all_p
.PHONY: clean distclean
.PHONY: install install_p
