/*
 * Copyright (c) 1999, 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * Define SAMERATE to put packets out equally on each port.
 * Undefine it to put out packets in an uneven ratio.
 */
#undef SAMERATE

/*
 * LBS means my home machine!
 */
#undef LBS

/*
 * Test the packet scheduler stuff. Send many packets very fast to a set of
 * UDP ports on another machine, which gets them and tells you the relative
 * percentages. At present, there is no way to specify the bandwidth parm,
 * so every outgoing channel gets the same ticket allocation. In the test
 * below, each of 3 channels gets different numbers of packets sent over it,
 * but the receiver should get exactly 33% on each link.
 *
 * The receiver program is the hpfq_udp_tally program that Bart wrote to test
 * the HPFQ stuff. Its actually a standalone program, but lives in the oskit
 * tree. Maybe it should move here?
 */
#include <stdio.h>
#include <assert.h>
#include <string.h>
#include <sys/types.h>

#include <sys/socket.h>
#include <net/if.h>	/* XXX solaris */
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netinet/in_systm.h>
#include <netinet/if_ether.h>
#include <netinet/ip.h>

#define __FAVOR_BSD	/* XXX get BSD field names in Linux udp.h */
#include <netinet/udp.h>
#undef __FAVOR_BSD	/* XXX but don't effect anything else */

#include <nodeos/an_impl.h>
#include <nodeos/an_flow.h>
#include <nodeos/an_thread.h>
#include <nodeos/an_chan.h>
#include <nodeos/an_mutex.h>
#include <nodeos/an_pbuf.h>
#include <nodeos/an_info.h>
#include <nodeos/an_event.h>

#include <nodeos/impl/an_gprof.h>	/* XXX */

#include <nodeos/lib/pbcache.h>
#include <nodeos/lib/utils.h>

#define RUNFOR		20	/* run for this many seconds */
#define MAXPORTS	3
#define NUMPBUFS	1500	/* per port */

#define SRC_PORT	10000
#define DST_PORT	5050
//#define PKT_SIZE	(64-(18+28))	/* ethernet min */
#define PKT_SIZE	(1518-(18+28))	/* ethernet max */

#ifdef LBS
#define SRC_IP		"206.163.153.26"                /* Poohbear, eth0 */
#define DST_IP		"206.163.153.25"                /* Golden, eth1 */
#define SRC_IF		0
#else
#define SRC_IP		"155.99.214.138"		/* tbpc38 */
//#define SRC_IP		"155.99.214.151"		/* chukar */
//#define DST_IP		"155.99.214.150"		/* falcon */
#define DST_IP		"155.99.214.74"			/* plastic */
#define SRC_IF		4
#endif

static unsigned char dst_ethaddr[6] = {
#ifdef LBS
	0x00,0xa0,0xc9,0xf1,0x09,0xcb	/* golden, eth1 */
#else
//	0x00,0xa0,0xc9,0x82,0x89,0x94	/* falcon, eth0 */
	0x00,0x90,0x27,0x8d,0x44,0xc0	/* plastic, fxp1 */
#endif
};

/*
 * For some reason the headers define ETHER_MAX_LEN as
 * (ETH_FRAME_LEN + ETH_CRC_LEN), but then don't define ETH_CRC_LEN.
 */
#ifdef __linux__
#ifndef ETH_CRC_LEN
#define ETH_CRC_LEN ETHER_CRC_LEN
#endif
#endif

static void ee_init(void *arg);
an_flowinit_func_t InitEE = { ee_init };

static void ee_die(void *event, void *arg);
static void setup_channels(void);
static void setup_buffers(int numbufs);
static void teardown_buffers(void);
static an_pbuf_t *get_pbuf(int which);
static void put_pbuf(int which, an_pbuf_t *pbuf);

static an_chan_t *ethoutchan[MAXPORTS], ethoutchan_data[MAXPORTS];

an_ethaddrspec_t myaddr = {
	{0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, {0}, ETHERTYPE_IP
};
an_ethaddrspec_t ipoutaddr = {
	{0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, {0}, ETHERTYPE_IP
};

static int dying, done;
static an_timespec_t startt, endt;

int
ee_main(int argc, const char **argv)
{
	an_pbuf_t *pbuf;
	int i;
	
	assert(argc >= 1);

	printf("%p/%p: %s started\n",
	       an_flow_current(), an_thread_current(), argv[0]);

	/*
	 * Setup channels
	 */
	setup_buffers(NUMPBUFS);
	setup_channels();

	printf("%s (%02x:%02x:%02x:%02x:%02x:%02x) ready\n",
	       SRC_IP,
	       myaddr.daddr[0], myaddr.daddr[1], myaddr.daddr[2],
	       myaddr.daddr[3], myaddr.daddr[4], myaddr.daddr[5]);

	an_gprof_start(argv[0]);
#ifdef OSKIT
	{
		extern int memlogging;
		memlogging = 1;
	}
#endif

	an_gettime(&startt);
#if 0
	for (i = 0; !dying && i < 500000; i++) {
		pbuf = get_pbuf(0);
		an_outchan_send(ethoutchan[0], pbuf);
		put_pbuf(0, pbuf);
	}
#else
	for (i = 0; !dying && i < 500000; i++) {
#ifdef SAMERATE
		/*
		 * Every channel has an equal share of the bandwidth.
		 * Transmit a *different* number of packets on every channel.
		 *
		 * If all is well, the receiver should see exactly 33% on
		 * each port, even though the number of packets sent on
		 * each port is very unequal.
		 */
		pbuf = get_pbuf(0);
		an_outchan_send(ethoutchan[0], pbuf);
		put_pbuf(0, pbuf);
		pbuf = get_pbuf(1);
		an_outchan_send(ethoutchan[1], pbuf);
		put_pbuf(1, pbuf);
		pbuf = get_pbuf(2);
		an_outchan_send(ethoutchan[2], pbuf);
		put_pbuf(2, pbuf);
		
		pbuf = get_pbuf(0);
		an_outchan_send(ethoutchan[0], pbuf);
		put_pbuf(0, pbuf);
		pbuf = get_pbuf(1);
		an_outchan_send(ethoutchan[1], pbuf);
		put_pbuf(1, pbuf);

		pbuf = get_pbuf(0);
		an_outchan_send(ethoutchan[0], pbuf);
		put_pbuf(0, pbuf);
#else
		/*
		 * Every channel has an unequal share of the bandwidth.
		 * Transmit the *same* number of packets on every channel.
		 *
		 * If all is well, the receiver should see an uneven ratio
		 * (1:2:3) of packets on the ports, event though the number
		 * of packets sent on each port is the same.
		 */
		pbuf = get_pbuf(0);
		an_outchan_send(ethoutchan[0], pbuf);
		put_pbuf(0, pbuf);
		pbuf = get_pbuf(1);
		an_outchan_send(ethoutchan[1], pbuf);
		put_pbuf(1, pbuf);
		pbuf = get_pbuf(2);
		an_outchan_send(ethoutchan[2], pbuf);
		put_pbuf(2, pbuf);
#endif
	}
#endif
	an_gettime(&endt);
	done = 1;
	return 0;
}

static void
setup_channels(void)
{
	an_error_t err;
	struct {
		an_chanaddr_t addr;
		an_chanproto_t proto;
		an_cpuspec_t cpu;
		an_netspec_t net;
	} attr;
	an_flow_t *flow;
	an_infoif_t info[8], *infop;
	int nif = 8;
	int i;

	flow = an_flow_current();
	memset(&attr, 0, sizeof attr);

	/*
	 * Find my address.
	 * XXX assumes first interface is ethernet.
	 */
	err = an_info_getifs(info, &nif);
	assert(err == 0);
	assert(nif >= SRC_IF);
	infop = &info[SRC_IF];
	assert(info[SRC_IF].type == ANIF_ETH);
	memcpy(myaddr.daddr, infop->addr.eth.daddr, sizeof myaddr.daddr);

	/*
	 * Setup an event to time us out in RUNFOR seconds
	 */
	err = an_event_schedule(ee_die, 0, RUNFOR * 1000000);
	assert(err == 0);
	printf("Timer started, running for %d seconds\n", RUNFOR);
	
	/*
	 * Setup raw ethernet output channels.
	 */
	for (i = 0; i < MAXPORTS; i++) {
		memset(&attr, 0, sizeof attr);
		strncpy(attr.proto.data, infop->pname, sizeof infop->pname);
		attr.proto.len = strlen(attr.proto.data);
		memset(&attr.addr, 0, sizeof attr.addr);
		attr.net.bandwidth = 100000;	/* XXX 100kbps */
#ifndef SAMERATE
		attr.net.bandwidth += (i * 100000);
#endif
		ethoutchan[i] = an_outchan_create(&ethoutchan_data[i], flow,
						  &attr.proto, &attr.addr,
						  &attr.net);
		assert(ethoutchan[i] != 0);
		printf("%p: eth%d outchannel\n", ethoutchan[i], SRC_IF);
	}
}

static void
ee_die(void *event, void *arg)
{
	int i, secs;

	if (!done)
		an_gettime(&endt);
	if ((secs = endt.tv_sec - startt.tv_sec) == 0)
		secs = 1;

	dying = 1;
	an_gprof_stop();
	printf("All done, actively transmitted packets for %d seconds\n", secs);

	for (i = 0; i < MAXPORTS; i++) {
		char buf[16];

		snprintf(buf, 16, "Chan%d", i);
		moab_chan_printstats(ethoutchan[i], 1, buf, secs, 1);
		an_outchan_destroy(ethoutchan[i]);
	}
	moab_chan_printstats(0, 1, "Total", secs, 0);
	moab_flow_printstats(an_flow_current(), 1, "Flow", secs, 0);
	teardown_buffers();
	an_flow_destroy(an_flow_current());
}

static void
ee_init(void *arg)
{
	int status;
	int argc = 1;
	const char *argv[] = { "PackSchedEE" }; 

	status = ee_main(argc, argv);
}

static void
setup_ether_packet(int which, char *p)
{
	struct ether_header	eth;
	struct ip		ip;
	struct udphdr		udp;
	int			i;
	struct in_addr		src_ip, dst_ip;
	static int		id;

	inet_aton(DST_IP, &dst_ip);
	inet_aton(SRC_IP, &src_ip);

	/*
	 * Set up ether header
	 */
	memcpy(&eth.ether_shost, myaddr.daddr, sizeof(myaddr.daddr));
	memcpy(&eth.ether_dhost, dst_ethaddr, sizeof(dst_ethaddr));
	eth.ether_type = htons(ETHERTYPE_IP);

	/*
	 * Setup UDP Header
	 */
	udp.uh_sport = htons(SRC_PORT + which);
	udp.uh_dport = htons(DST_PORT);
	udp.uh_ulen  = htons(sizeof(struct udphdr) + PKT_SIZE);
	udp.uh_sum   = 0;

	/*
	 * Setup IP header
	 */
	ip.ip_v		 = IPVERSION;
	ip.ip_hl	 = (sizeof ip) >> 2;
	ip.ip_tos	 = 0;
	ip.ip_len	 = htons(sizeof(struct ip) +
				 sizeof(struct udphdr) + PKT_SIZE);
	ip.ip_id	 = ++id;	/* apparently can't leave this
					  zero or we get dropped by FreeBSD  */
	ip.ip_off	 = 0;
	ip.ip_ttl	 = 64;
	ip.ip_p		 = IPPROTO_UDP;
	ip.ip_src.s_addr = src_ip.s_addr;	/* already in net order */
	ip.ip_dst.s_addr = dst_ip.s_addr;	/* already in net order */
	ip.ip_sum        = 0;
	ip.ip_sum	 = ipcksum(&ip, sizeof(ip));

	/*
	 * Now copy into packet to avoid alignment breakage on ARM32
	 */
	memcpy(p, &eth, sizeof(struct ether_header));
	p += sizeof(struct ether_header);
	
	memcpy(p, &ip, sizeof(struct ip));
	p += sizeof(struct ip);

	memcpy(p, &udp, sizeof(struct udphdr));
	p += sizeof(struct udphdr);

	/*
	 * And fill in some data for the payload.
	 */
	for (i = 0; i < PKT_SIZE; i++)
		*p++ = i;
}

/*
 * Packet buffer management
 */

static pbcache_t *cache[MAXPORTS];
static int curcache;

static void
init_buffer(an_pbuf_t *pbuf)
{
	setup_ether_packet(curcache, an_pbuf_data(pbuf));
	an_pbuf_setsize(pbuf, sizeof(struct ether_header) +
			sizeof(struct ip) + sizeof(struct udphdr) +
			PKT_SIZE, 0);
}

static void
setup_buffers(int nbufs)
{
	int i;

	for (i = 0; i < MAXPORTS; i++) {
		curcache = i;
		cache[i] = pbcache_create(NUMPBUFS, init_buffer);
		assert(cache[i] != 0);
	}
}

static void
teardown_buffers(void)
{
	int i, n;

	for (i = 0; i < MAXPORTS; i++) {
		printf("Chan%d pbcache: ", i);
		pbcache_dump(cache[i]);
		n = pbcache_destroy(cache[i], 0);
		if (n != 0)
			printf("* * * %d buffers not destroyed\n", n);
	}
}

static an_pbuf_t *
get_pbuf(int which)
{
	return pbcache_alloc(cache[which], 1);
}

static void
put_pbuf(int which, an_pbuf_t *pbuf)
{
	pbcache_free(cache[which], pbuf);
}
