#
# Copyright (c) 1999, 2000 University of Utah and the Flux Group.
# All rights reserved.
# This file is part of Janos.
# 
# Janos is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License, version 2, as
# published by the Free Software Foundation.
# 
# Janos is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details, in the file
# COPYING.  If that file is not present, write to the Free
# Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
# 02111-1307, USA
# 
# The University of Utah and the Flux Group requests users to
# return to flux-dist@cs.utah.edu any improvements that they make.
#

TARGET =	libmoabutil.a
TARGET_P =	libmoabutil_p.a

SRCS =		arplib.c pbcache.c ipcksum.c \
		chanstats.c flowstats.c mpoolstats.c configfile.c

vpath %.c $(SRCDIR)

include $(NODEOS_SRCDIR)/arch.mk

FILES = 	$(SRCS)
FQSRCS =	$(addprefix $(SRCDIR)/,$(sort $(FILES)))
DEPFILES:=	$(patsubst %.c,%.d,$(SRCS) $(ARCH_SRCS))

OBJS := $(patsubst %.c,%.o,$(notdir $(FQSRCS)))
POBJS := $(patsubst %.c,%.po,$(notdir $(FQSRCS)))

#CFLAGS_WARNINGS=-W -Wcast-align -Wwrite-strings -Wstrict-prototypes -Wmissing-prototypes -Wnested-externs -Winline
CFLAGS_WARNINGS = -Wwrite-strings -Wstrict-prototypes -Winline

ifneq ($(HOST_OS),solaris)	# these are too annoying on solaris
CFLAGS_WARNINGS += -Wcast-align -Wshadow
endif

CPPFLAGS = $(OPT_CFLAGS) $(ARCH_CFLAGS) -I../ -I$(NODEOS_SRCDIR)
CFLAGS += -Wall $(CFLAGS_WARNINGS) $(CPPFLAGS)

ifeq ($(ENABLE_PROFILING),yes)
all: all_ all_p
install: install_ install_p
else
all: all_
install: install_
endif

all_: $(TARGET)

all_p: $(TARGET) $(TARGET_P)

install_dirs:
	@test -d $(INSTALL_TOPDIR) || mkdir $(INSTALL_TOPDIR)
	@test -d $(INSTALL_LIBDIR) || mkdir $(INSTALL_LIBDIR)

install_: install_dirs $(TARGET)
	$(SILENT_MAKE_INFORM) "Installing $(TARGET)."
	$(INSTALL_DATA) $(TARGET) $(INSTALL_LIBDIR)

install_p: install_dirs $(TARGET_P)
	$(SILENT_MAKE_INFORM) "Installing $(TARGET_P)."
	$(INSTALL_DATA) $(TARGET_P) $(INSTALL_LIBDIR)


$(TARGET): $(OBJS)
	$(SILENT_MAKE_INFORM) "Creating library $@."
	$(AR) cru $@ $^
	$(RANLIB) $@

$(TARGET_P): $(POBJS)
	$(SILENT_MAKE_INFORM) "Creating library $@."
	$(AR) cru $@ $^
	$(RANLIB) $@

clean:
	$(SILENT_MAKE_INFORM) "Removing *.o *.po $(TARGET) $(TARGET_P)."
	rm -f .depend core *.core *.d *.o *.po $(TARGET) $(TARGET_P)

distclean: clean
	$(SILENT_MAKE_INFORM) "Scrubbing nodeos library directory."
	rm -f GNUmakefile

depend: 
	@echo "WARNING: depend target is deprectated."

mkdebug:
	@echo "ARCH_SRCS = $(ARCH_SRCS)"
	@echo "DEPFILES  = $(DEPFILES)"

-include $(DEPFILES)

.PHONY: depend distclean clean mkdebug
.PHONY: all all_ all_p
.PHONY: install install_dirs install_ install_p 

