/*
 * Copyright (c) 1999, 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
#include <stdio.h>
#include <nodeos/an_mempool.h>

static an_mempoolstats_t totstats;
static void print_stats(an_mempoolstats_t *stats, int verbose, const char *str);
static void add_stats(an_mempoolstats_t *stats);

an_error_t
moab_mempool_printstats(an_mempool_t *mpool, int verbose, const char *str,
			int accum)
{
	an_mempoolstats_t stats;
	an_error_t err;
	
	if (mpool == 0) {
		print_stats(&totstats, verbose, str);
		memset(&totstats, 0, sizeof totstats);
		return 0;
	}

	err = an_mempool_stats(mpool, &stats);
	if (err == 0) {
		print_stats(&stats, verbose, str);
		if (accum)
			add_stats(&stats);
	}

	return err;
}

static void
print_stats(an_mempoolstats_t *stats, int verbose, const char *str)
{
	unsigned long long totmem;

	totmem = stats->exchunks + stats->imchunks;
	if (totmem == 0) {
		if (verbose)
			printf("%s: no memory\n", str);
		return;
	}

	printf("%s: %d flows using %qu bytes of memory\n",
	       str, (int)stats->numflows, totmem * stats->chunksize);

	if (verbose)
		printf("  %qu explicit, %qu implicit\n",
		       stats->exchunks, stats->imchunks);
}

static void
add_stats(an_mempoolstats_t *stats)
{
	totstats.exchunks += stats->exchunks;
	totstats.imchunks += stats->imchunks;
	totstats.chunksize = stats->chunksize;
	totstats.numflows += stats->numflows;
}
