/*
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * NodeOS Event API
 */
#ifndef _NODEOS_EVENT_H_
#define _NODEOS_EVENT_H_

#ifdef NODEOS_SPEC_COMPAT
#define an_eventSchedule an_event_schedule
#endif

#include <nodeos/an_error.h>
#include <nodeos/an_impl.h>

/*
 * Schedule an event to occur at least the indicated number of microseconds
 * in the future.  The given handler is run in the context of the current
 * flow and is provided with the given argument.
 *
 * ``func'' is the function to invoke when the specified time has elapsed,
 * ``farg'' is the argument for the handler function,
 * ``delta'' is the time (in usec) in the future at which to run the handler.
 *
 * Returns zero if the event has been scheduled, an error otherwise.
 */
an_error_t an_event_schedule(an_eventfunc_t func, void *farg,
			     unsigned long delta);

/*
 * Return the granularity (in microseconds) of the underlying event timer.
 * Scheduled events will be rounded up to the nearest multiple of this value.
 */
unsigned long an_event_granularity(void);

#endif /* _NODEOS_EVENT_H_ */
