/*
 * Copyright (c) 1999, 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * NodeOS Flow API
 */
#ifndef _NODEOS_FLOW_H_
#define _NODEOS_FLOW_H_

#ifdef NODEOS_SPEC_COMPAT
#define an_Flow an_flow_t
#define an_FlowInitFunc an_flowinit_func_t
#define an_FlowTermFunc an_flowterm_func_t
#define an_FlowStats an_flowstats_t
#define an_flowCreate an_flow_create
#define an_flowDestroy an_flow_destroy
#define an_flowId an_flow_current
#define an_flowStartThread an_flow_startthread
#define an_flowStats an_flow_stats
#endif

#include <nodeos/an_error.h>
#include <nodeos/an_impl.h>
#include <nodeos/an_cred.h>
#include <nodeos/an_resource.h>
#include <nodeos/impl/an_flow.h>

/*
 * Create a new flow with the indicated credentials and resources.
 * When the NodeOS has finished setting up the internal state of the flow,
 * a flow thread is used to asynchronously invoke the provided init function;
 * i.e., the create call may return before the init functions completes or
 * is even started.  The provided termination handler will be invoked when
 * terminating the flow, see an_flow_destroy.  Returns the new flow object
 * or zero on error.
 */
an_flow_t *an_flow_create(void *mem, an_cred_t *cred,
			  an_cpuspec_t *cpu, an_memspec_t *vm,
			  an_flowinit_func_t init, void *initarg,
			  an_flowterm_func_t term);

/*
 * Gracefully shutdown the indicated flow.
 * Invokes the registered termination callback synchronously.  Upon return
 * from that callback, internal flow destruction commences.  First, all
 * sub-flows are "internally" destroyed (i.e., no termination callbacks
 * for subflows).  Any remaining flow threads are forcibly destroyed and
 * all flow resources are freed.
 */
an_error_t an_flow_destroy(an_flow_t *flow);

/*
 * Returns the flow identifier associated with the currently running thread.
 */
an_flow_t *an_flow_current(void);

/*
 * Activate a flow thread at a specified function.
 * The new thread starts in the indicated function with the indicated argument.
 * Returns zero if the thread launched, an error otherwise.  If out_thread is
 * non-zero, returns a pointer to the new thread there.
 */
struct an_thread;
an_error_t an_flow_startthread(an_flow_t *flow, void (*f)(void *arg), void *arg,
			       struct an_thread **out_thread);

/*
 * Return statistics for a flow.
 * Return thread and other statistics for the flow in the provided buffer.
 * Returns zero on success or an error code otherwise.
 */
an_error_t an_flow_stats(an_flow_t *flow, an_flowstats_t *stats);

#endif /* _NODEOS_FLOW_H_ */
