/*
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * Moab Setup API
 */
#ifndef _NODEOS_MOAB_SETUP_H_
#define _NODEOS_MOAB_SETUP_H_

#ifdef NODEOS_SPEC_COMPAT
#define an_moabSetup an_moab_setup
#define an_moabVersion an_moab_version
#define an_moabStart an_moab_start
#endif

#include <nodeos/an_error.h>
#include <nodeos/an_impl.h>
#include <nodeos/an_cred.h>
#include <nodeos/an_flow.h>
#include <nodeos/an_mempool.h>

#include <nodeos/impl/an_moab_setup.h>

/*
 * Initialize the underlying device layers.
 */
void an_moab_setup(void);

/*
 * Return the compiled version of Moab, as an integer.
 */
int an_moab_version(void);

/*
 * Start a new thread in the root flow on its way.  The
 * initial objects are all uninitialized.
 *
 * DOES NOT RETURN UNTIL MOAB IS SHUTDOWN.
 */
void an_moab_start(an_flow_t *root_flow, an_cred_t *root_cred,
		   an_mempool_t *root_mempool, int nthreads, int stacksize,
		   void **root_threads, void **root_threadstacks,
		   void (*initee_func)(void *), void *initee_arg);

/*
 * Shutdown Moab.  This signals the thread that started moab via
 * an_moab_start, and causes the root flow and all its children
 * to be terminated.
 */
void an_moab_shutdown(void);


#endif /* _NODEOS_MOAB_SETUP_H_ */
