/*
 * Copyright (c) 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * Implementation bits of NodeOS cred API.
 */
#ifndef _NODEOS_IMPL_AN_CRED_H_
#define _NODEOS_IMPL_AN_CRED_H_

#include <nodeos/impl/decls.h>
#include <nodeos/impl/ani_cred.h>
#include <nodeos/impl/ani_nodeos.h>

/*
 * an_cred interfaces.  
 *
 * an_creds are a simple wrapper around the ani_cred layer.
 * This layer add the NODEOS entry/exit and INTERN/EXTERN to the ani_cred.
 *
 * Currently we only inline an_cred_current.
 */

typedef struct an_cred {
	ani_cred_t cred;
} an_cred_t;

/* an_cred inline functions */

NODEOS_FUNC_INLINE ani_cred_t *
NODEOS_INTERN_CRED(an_cred_t *xcred)
{
	assert(xcred != 0);
	return &xcred->cred;
}

NODEOS_FUNC_INLINE an_cred_t *
NODEOS_EXTERN_CRED(ani_cred_t *icred)
{
	assert(icred != 0);
	return (an_cred_t *)icred;
}

NODEOS_FUNC_INLINE an_cred_t *
an_cred_current(void)
{
	an_cred_t *cred;

	LW_NODEOS_ENTER;
	cred = NODEOS_EXTERN_CRED(ani_cred_current());
	LW_NODEOS_LEAVE;
	return cred;
}

#endif /* _NODEOS_IMPL_AN_CRED_H_ */
