/*
 * Copyright (c) 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * Implementation bits of NodeOS flow API.
 */
#ifndef _NODEOS_IMPL_AN_FLOW_H_
#define _NODEOS_IMPL_AN_FLOW_H_

#include <nodeos/impl/decls.h>
#include <nodeos/impl/ani_nodeos.h>
#include <nodeos/impl/ani_flow.h>

/*
 * an_flow interfaces.  
 *
 * an_flows are a simple wrapper around the ani_flow layer.
 * This layer add the NODEOS entry/exit and INTERN/EXTERN to the ani_flow.
 *
 * Currently we only inline an_flow_current.
 */

typedef struct an_flow {
	ani_flow_t flow;
} an_flow_t;

#define AN_SIZEOF_FLOW	sizeof(an_flow_t)

/* an_flow inline functions */

NODEOS_FUNC_INLINE ani_flow_t *
NODEOS_INTERN_FLOW(an_flow_t *xflow)
{
	assert(xflow != 0);
	return &xflow->flow;
}

NODEOS_FUNC_INLINE an_flow_t *
NODEOS_EXTERN_FLOW(ani_flow_t *iflow)
{
	assert(iflow != 0);
	return (an_flow_t *)iflow;
}

NODEOS_FUNC_INLINE an_flow_t *
an_flow_current(void)
{
	an_flow_t *flow;

	LW_NODEOS_ENTER;
	flow = NODEOS_EXTERN_FLOW(ani_flow_current());
	LW_NODEOS_LEAVE;
	return flow;
}

#endif /* _NODEOS_IMPL_AN_FLOW_H_ */
