/*
 * Copyright (c) 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

#include <nodeos/impl/ani_nodeos.h>
#include <nodeos/impl/ani_fs.h>

#ifndef _NODEOS_IMPL_AN_FS_H_
#define _NODEOS_IMPL_AN_FS_H_

NODEOS_FUNC_INLINE an_error_t 
an_file_close(an_file_t *file) 
{
	an_error_t error;

	NODEOS_ENTER;
	error =	 ani_file_close(file);
	NODEOS_LEAVE;
	return error;
}

NODEOS_FUNC_INLINE an_error_t 
an_file_fsync(an_file_t *file)
{
	an_error_t error;

	NODEOS_ENTER;
	error =	 ani_file_fsync(file);
	NODEOS_LEAVE;
	return error;
}

NODEOS_FUNC_INLINE an_error_t 
an_file_truncate(an_file_t *file, an_offset_t offset)
{
	an_error_t error;

	NODEOS_ENTER;
	error =	 ani_file_truncate(file, offset);
	NODEOS_LEAVE;
	return error;
}

NODEOS_FUNC_INLINE an_error_t 
an_file_lseek(an_file_t *file, an_offset_t length, an_whence_t whence,
	      an_offset_t *retoff)
{
	an_error_t error;

	NODEOS_ENTER;
	error =	 ani_file_lseek(file, length, whence, retoff);
	NODEOS_LEAVE;
	return error;
}

NODEOS_FUNC_INLINE an_error_t 
an_file_mkdir(const char *path, an_acd_t *acd) 
{
	an_error_t error;

	NODEOS_ENTER;
	error =	 ani_file_mkdir(path, acd);
	NODEOS_LEAVE;
	return error;
}

NODEOS_FUNC_INLINE an_error_t 
an_file_read(an_file_t *file, an_size_t nbytes, void *buf, an_size_t *retlen)
{
	an_error_t error;

	NODEOS_ENTER;
	error =	 ani_file_read(file, nbytes, buf, retlen);
	NODEOS_LEAVE;
	return error;
}

NODEOS_FUNC_INLINE an_error_t 
an_file_rename(const char *from, const char *to)
{
	an_error_t error;

	NODEOS_ENTER;
	error =	 ani_file_rename(from, to);
	NODEOS_LEAVE;
	return error;
}

NODEOS_FUNC_INLINE an_error_t 
an_file_write(an_file_t *file, an_size_t length, void *buf, an_size_t *retval)
{
	an_error_t error;

	NODEOS_ENTER;
	error = ani_file_write(file, length, buf, retval);
	NODEOS_LEAVE;
	return error;
}

NODEOS_FUNC_INLINE an_error_t 
an_file_unlink(const char *path)
{
	an_error_t error;

	NODEOS_ENTER;
	error = ani_file_unlink(path);
	NODEOS_LEAVE;
	return error;
}

NODEOS_FUNC_INLINE an_error_t 
an_file_open(const char *path, int flags, an_acd_t *acd, an_file_t *retval)
{
	an_error_t error;

	NODEOS_ENTER;
	error = ani_file_open(path, flags, acd, retval);
	NODEOS_LEAVE;
	return error;
}

NODEOS_FUNC_INLINE an_error_t 
an_file_stat(const char *path, an_stat_t *retval)
{
	ani_error_t error;

	NODEOS_ENTER;
	error = ani_file_stat(path, retval);
	NODEOS_LEAVE;
	return NODEOS_EXTERN_ERROR(error);
}

NODEOS_FUNC_INLINE an_error_t 
an_file_fstat(an_file_t *file, an_stat_t *retval)
{
	ani_error_t error;

	NODEOS_ENTER;
	error = ani_file_fstat(file, retval);
	NODEOS_LEAVE;
	return NODEOS_EXTERN_ERROR(error);
}

#endif
