/*
 * Copyright (c) 1999, 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

#ifndef _NODEOS_IMPL_MOAB_SETUP_H_
#define _NODEOS_IMPL_MOAB_SETUP_H_

#include <nodeos/impl/ani_moab_setup.h>

NODEOS_FUNC_INLINE void 
an_moab_setup(void)
{
	/* No context, cannot call NODEOS_ENTER or NODEOS_EXIT */

	ani_moab_setup();
}

NODEOS_FUNC_INLINE int 
an_moab_version(void)
{
	int ver;

	LW_NODEOS_ENTER;
	ver = ani_moab_version();
	LW_NODEOS_LEAVE;
	
	return ver;
}

NODEOS_FUNC_INLINE void 
an_moab_start(an_flow_t *root_flow, an_cred_t *root_cred,
	      an_mempool_t *root_mempool, int nthreads, int stacksize,
	      void **root_threads, void **root_threadstacks,
	      void (*initee_func)(void *), void *initee_arg)
{
	/*
	 * No context, cannot call NODEOS_ENTER or NODEOS_EXIT
	 *
	 * N.B. No copyin of thread/stack arrays necessary in our impl.
	 */
	ani_moab_start(NODEOS_INTERN_FLOW(root_flow),
		       NODEOS_INTERN_CRED(root_cred),
		       NODEOS_INTERN_MEMPOOL(root_mempool),
		       nthreads, stacksize,
		       root_threads,
		       root_threadstacks,
		       initee_func,
		       initee_arg);
}

NODEOS_FUNC_INLINE void 
an_moab_shutdown(void)
{
	/*
	 * We may or may not have a context here, but it doesn't matter,
	 * the world is about to end so we don't bother with ENTER/LEAVE
	 */
	ani_moab_shutdown();
}


#endif /* _NODEOS_MOAB_SETUP_H_ */
