/*
 * Copyright (c) 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * Implementation bits of NodeOS thread API.
 */
#ifndef _NODEOS_IMPL_AN_THREAD_H_
#define _NODEOS_IMPL_AN_THREAD_H_

#include <nodeos/impl/decls.h>
#include <nodeos/impl/ani_nodeos.h>
#include <nodeos/impl/ani_thread.h>

/*
 * an_thread interfaces.  
 *
 * an_threads are a simple wrapper around the ani_thread layer.
 * This layer add the NODEOS entry/exit and INTERN/EXTERN to the ani_thread.
 *
 * Currently we only inline an_thread_current.
 */

typedef struct an_thread {
	ani_thread_t thread;
} an_thread_t;

/* an_thread inline functions */

void an_thread_exit(void) NODEOS_FUNC_NORETURN;

NODEOS_FUNC_INLINE ani_thread_t *
NODEOS_INTERN_THREAD(an_thread_t *xthread)
{
	assert(xthread != 0);
	return &xthread->thread;
}

NODEOS_FUNC_INLINE an_thread_t *
NODEOS_EXTERN_THREAD(ani_thread_t *ithread)
{
	assert(ithread != 0);
	return (an_thread_t *)ithread;
}

NODEOS_FUNC_INLINE an_thread_t *
an_thread_current(void)
{
	an_thread_t *thread;

	LW_NODEOS_ENTER;
	thread = NODEOS_EXTERN_THREAD(ani_thread_current());
	LW_NODEOS_LEAVE;
	return thread;
}

#endif /* _NODEOS_IMPL_AN_THREAD_H_ */
