/*
 * Copyright (c) 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
#ifndef _NODEOS_IMPL_ANI_SYSCOND_H_
#define _NODEOS_IMPL_ANI_SYSCOND_H_

/*
 * ani_cond_*
 *
 * ani_cond vars wrap the ani_icond.  The wait and signal/broadcast
 * interfaces provide atomicity through the ilock in the ani_cond.
 *
 * Currently none of the ani_cond variables are inlined.
 */

#include <nodeos/an_time.h>
#include <nodeos/impl/ani_error.h>
#include <nodeos/impl/ani_ilock.h>
#include <nodeos/impl/ani_mutex.h>

typedef struct ani_cond_t {
	ani_icond_t	icond;
	/* 
	 * This lock protects the condition variable
	 * while the user-associated lock is released.
	 */
	ani_ilock_t	ilock;
} ani_cond_t;

/* prototypes */

ani_error_t ani_cond_init(ani_cond_t *newcond);

void ani_cond_timedwait(ani_cond_t *cond, ani_mutex_t *mutex,
			   an_timespec_t *abstime);

void ani_cond_signal(ani_cond_t *cond);

void ani_cond_broadcast(ani_cond_t *cond);

ani_error_t ani_cond_deinit(ani_cond_t *cond);


/* inline functions */

NODEOS_FUNC_INLINE void
ani_cond_wait(ani_cond_t *cond, ani_mutex_t *mutex)
{
	/* just call timedwait with a NULL timespec. */
	ani_cond_timedwait(cond, mutex, NULL);
}

NODEOS_FUNC_INLINE void
ani_cond_lock(ani_cond_t *cond)
{
	ani_ilock_lock(&(cond->ilock));
}

NODEOS_FUNC_INLINE int
ani_cond_trylock(ani_cond_t *cond)
{
	return ani_ilock_trylock(&(cond->ilock));
}

NODEOS_FUNC_INLINE void
ani_cond_unlock(ani_cond_t *cond)
{
	ani_ilock_unlock(&(cond->ilock));
}

#endif /* _NODEOS_IMPL_ANI_SYSCOND_H_ */
