/*
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * Credentials: as of yet ill-defined
 *
 * XXX dealing with resources here may be way off base.
 */
#ifndef _CRED_IMPL_H_
#define _CRED_IMPL_H_

#include <nodeos/impl/ani_error.h>
#include <nodeos/impl/ani_resource.h>

struct ani_cred {
	void		*stuff;		/* XXX authentication stuff */
	ani_resource_t	current;	/* resources allotted to this "user" */
	ani_resource_t 	limits;		/* resource limits for this "user" */
};
typedef struct ani_cred ani_cred_t;

/* credential_check operations */
#define CCHECK_FLOW_CREATE	1

/* credential_check returns */
#define CCHECK_OK	0
#define CCHECK_FAIL	1

/*
 * The root credential is speical.
 *
 * No resource limit checks are performed for flows associated with this
 * credential.  The root credential is generally used only internal to the
 * NodeOS for its root flow so that it can create ``daemon'' threads of the
 * highest priority.
 *
 * The resource limits of the root credential still correctly reflect the
 * amount of resources not allocated to other credentials.  In this way we
 * can still derive multiple credentials from the root credential without
 * exceeding the total resources available in the node.
 *
 * The current resource values are always zero for the root credential.
 */
ani_cred_t *ani_root_credential;

NODEOS_FUNC_INLINE void assert_ani_cred_valid(ani_cred_t *cred);
void ani_cred_setup(ani_cred_t *root_cred, ani_resource_t *noderes);
ani_error_t ani_cred_init(ani_cred_t *cred, void *auth, ani_resource_t *res);
void ani_cred_deinit(ani_cred_t *cred);
int ani_cred_check(ani_cred_t *cred, int op, ...);
ani_error_t ani_cred_resource_alloc(ani_cred_t *cred, ani_resource_t *res); 
void ani_cred_resource_free(ani_cred_t *cred, ani_resource_t *res);
ani_error_t ani_cred_resource_get(ani_cred_t *cred, unsigned int *nthreads,
				  unsigned int *cpurate, unsigned int *memtotal,
				  unsigned int *netrate);
ani_error_t ani_cred_resource_xfer(ani_cred_t *srccred, ani_cred_t *dstcred,
				   unsigned int nthreads, unsigned int cpurate,
				   unsigned int memtotal, unsigned int netrate);
struct ani_cred *ani_cred_current(void);

NODEOS_FUNC_INLINE void
assert_ani_cred_valid(ani_cred_t *cred) 
{
	assert(cred != 0);

#ifdef DEBUG
	if (cred != ani_root_credential) {
		extern ani_resource_t ani_moab_noderes;

		assert(cred->limits.threads <= ani_moab_noderes.threads);
		assert(cred->limits.cpurate <= ani_moab_noderes.cpurate);
		assert(cred->limits.timeslice <= ani_moab_noderes.timeslice);
		assert(cred->limits.stacksize <= 128 * 1024);
		assert(cred->limits.memtotal <= ani_moab_noderes.memtotal);

		/* Current usage should be below limits */
		assert(cred->current.threads <= cred->limits.threads);
		assert(cred->current.cpurate <= cred->limits.cpurate);
		assert(cred->current.timeslice <= cred->limits.timeslice);
		assert(cred->current.stacksize == 0
		       || cred->current.stacksize > 20*1024);	/* XXX oskit */
		assert(cred->current.stacksize <= cred->limits.stacksize);
		assert(cred->current.memtotal <= cred->limits.memtotal);
	} else {
		assert(cred->current.threads == 0);
		assert(cred->current.cpurate == 0);
		assert(cred->current.timeslice == 0);
		assert(cred->current.stacksize == 0);
		assert(cred->current.memtotal == 0);
	}
#endif /* DEBUG */
}

#endif /* _CRED_IMPL_H_ */
