/*
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

#ifndef _NODEOS_IMPL_ANI_EVENT_H_
#define _NODEOS_IMPL_ANI_EVENT_H_

#include <nodeos/impl/an_impl.h>
#include <nodeos/impl/an_error.h>
#include <nodeos/impl/ani_threadwork.h>

#define ANI_EVENT_RESOLUTION

struct ani_threadpool;

struct ani_event {
	struct ani_event *next;	/* next event in the event list */
	struct ani_threadpool *tpool;/* threadpool context for event */
	ani_threadwork_t work;	/* thread work record */
	int		 flags; /* event flags */
	an_eventfunc_t	 func;	/* handler */
	void		 *farg;	/* handler argument */
	unsigned long	 ticks; /* time in NodeOS tick at which to fire */
	unsigned long	 freq;	/* frequency in ticks for periodic events */
};
typedef struct ani_event ani_event_t;

#define ANI_EV_INKERN	0x01	/* exectute in the nodeos */
#define ANI_EV_PERIODIC	0x02	/* event should be rescheduled */
#define ANI_EV_DIRECT	0x04	/* call directly from the event loop */

void ani_event_setup(void);
void ani_event_shutdown(void);
void ani_event_info(unsigned long *timerres);
ani_error_t ani_event_init(ani_event_t *event, struct ani_threadpool *tpool,
			   int flags, an_eventfunc_t func, void *farg,
			   unsigned long delta);
ani_error_t ani_event_deinit(ani_event_t *event);
void ani_event_removeall(struct ani_threadpool *tpool);
void ani_event_dump(char *str);

#endif /* _NODEOS_IMPL_ANI_EVENT_H_ */
