/*
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * Resource descriptor
 */
#ifndef _NODEOS_IMPL_RESOURCE_H_
#define _NODEOS_IMPL_RESOURCE_H_

#include <limits.h>

typedef struct ani_resource {
	/* CPU */
	unsigned int threads;	/* number of threads */
	unsigned int timeslice;	/* time slice of threads in usec */
	unsigned int stacksize;	/* stack size in bytes */
	unsigned int cpurate;	/* rate of CPU use in cycles/sec */

	/* memory */
	unsigned int memtotal;	/* bytes of memory */

	/* network */
	unsigned int netrate;	/* rate of network bandwidth use in bits/sec */
} ani_resource_t;

#define RESOURCE_ALL	(INT_MAX)

/*
 * Determine the maximum available resources on the node
 */
void ani_resource_setup(ani_resource_t *noderes);

/*
 * Initializes part of the indicated resource structure based on the given
 * specification.  These routines perform any conversion of EE-visible
 * external units to internal representations; e.g., converting generic
 * CPU cycles/second to stride-scheduling tickets.
 */
struct an_cpuspec;
struct an_memspec;
struct an_netspec;
void ani_resource_initcpu(ani_resource_t *res, struct an_cpuspec *cspec);
void ani_resource_initmem(ani_resource_t *res, struct an_memspec *mspec);
void ani_resource_initnet(ani_resource_t *res, struct an_netspec *nspec);

#endif /* _RESOURCE_IMPL_H_ */
