/*
 * Copyright (c) 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

#ifndef _ANI_THREADWORK_H_
#define _ANI_THREADWORK_H_

/*
 * A thread work record.
 * Embedded in various objects that require threads do work for them;
 * e.g., pbuffers and events and, of course, threads themselves.
 */

struct ani_chan;

typedef void (*ani_thread_func_t)(void *, void *, void *);

struct ani_threadwork {
	struct ani_threadwork *next;	/* next item on list */
	int inkern;		/* function should run in kernel context */
	struct ani_chan *chan;	/* channel to associate thread with */
	ani_thread_func_t func;	/* function the thread should start in */
	void *farg[3];		/* and arguments for that function */
};
typedef struct ani_threadwork ani_threadwork_t;

NODEOS_FUNC_INLINE void
ani_threadwork_set(ani_threadwork_t *w, int inkern, struct ani_chan *chan,
		  ani_thread_func_t func, void *farg1, void *farg2, void *farg3)
{
	assert(func != 0);

	w->next = 0;
	w->inkern = inkern;
	w->chan = chan;
	w->func = func;
	w->farg[0] = farg1;
	w->farg[1] = farg2;
	w->farg[2] = farg3;
}

#endif /* _ANI_THREADWORK_H_ */
