/*
 * Copyright (c) 1999, 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * Initialize ARP goo.
 */
int		arp_init(void);

/*
 * Cleanup ARP goo (call a exit to kill numerous channels, buffers, etc).
 */
int		arp_exit(void);

/*
 * Configure an interface to do ARP requests and replies. The IP address
 * for the interface is given in network order. Subsequent to this call,
 * ARP requests for this IP address will be answered. ARP requests for
 * other IP addresses on this interface may also be issued (see the
 * arp_arpfor() call below).
 */
int		arp_config_iface(int iface, struct in_addr *ipaddr);

/*
 * Arp for an address on a particular interface. ipaddr must be in
 * network order. Thread blocks until there is a response, or until
 * the arp request times out (currently, a very adhoc timeout strategy
 * is employed). The hardware MAC address is copied into "ethaddr."
 */
int		arp_arpfor(int iface, struct in_addr *ipaddr,
			   unsigned char *ethaddr);

/*
 * Fill in MAC address for a local interface.
 */
int		arp_find_ethaddr(int iface, an_ethaddrspec_t *myaddr);

/*
 * Simple ARP cache routines
 */
int arp_cache_add(int iface, struct in_addr *ipaddr, unsigned char *ethaddr);
int arp_cache_remove(int iface, struct in_addr *ipaddr, unsigned char *ethaddr);
int arp_cache_lookup(int iface, struct in_addr *ipaddr, unsigned char *ethaddr);
