/*
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * The Credential API
 * XXX largely unspecified
 */
#include <nodeos/an_cred.h>
#include <nodeos/an_impl.h>

#include <nodeos/impl/ani_nodeos.h>
#include <nodeos/impl/ani_cred.h>
#include <nodeos/impl/ani_flow.h>

an_cred_t *
an_cred_create(void *mem)
{
	ani_cred_t *cred;
	ani_mempool_t *cpool;
	ani_error_t err;
	int didalloc = 0;

	NODEOS_ENTER;

	/*
	 * Allocate memory for the object if needed.
	 */
	cpool = ani_flow_mempool(ani_flow_current());
	if (mem == 0) {
		mem = ani_mempool_alloc(cpool, sizeof(an_cred_t));
		didalloc = 1;
	}

	cred = NODEOS_INTERN_CRED(mem);
	if (cred == 0)
		err = ani_error_set(AN_ERR_INVALIDARG,
				    "invalid an_cred argument");
	else
		err = ani_cred_init(cred, 0, 0);

	if (ani_error_occured(err) && didalloc)
		ani_mempool_free(cpool, mem, sizeof(an_cred_t));

	NODEOS_LEAVE;
	return ani_error_occured(err) ? 0 : NODEOS_EXTERN_CRED(cred);
}

an_error_t
an_cred_resource_get(an_cred_t *cred,
		     unsigned int *nthreads, unsigned int *cpurate,
		     unsigned int *memtotal, unsigned int *netrate)
{
	ani_error_t err;
	ani_cred_t *icred;

	NODEOS_ENTER;
	icred = NODEOS_INTERN_CRED(cred);
	if (icred == 0)
		err = ani_error_set(AN_ERR_INVALIDARG,
				    "invalid an_cred argument");
	else
		/*
		 * N.B. no copyout needed in our implementation
		 */
		err = ani_cred_resource_get(icred, nthreads, cpurate, memtotal,
					    netrate);

	NODEOS_LEAVE;
	return NODEOS_EXTERN_ERROR(err);
}

an_error_t
an_cred_resource_xfer(an_cred_t *srccred, an_cred_t *dstcred,
		      unsigned int nthreads, unsigned int cpurate,
		      unsigned int memtotal, unsigned int netrate)
{
	ani_error_t err;
	ani_cred_t *isrccred, *idstcred;

	NODEOS_ENTER;
	isrccred = NODEOS_INTERN_CRED(srccred);
	idstcred = NODEOS_INTERN_CRED(dstcred);
	if (isrccred == 0 || idstcred == 0)
		err = ani_error_set(AN_ERR_INVALIDARG,
				    "invalid an_cred argument");
	else
		err = ani_cred_resource_xfer(isrccred, idstcred, nthreads,
					     cpurate, memtotal, netrate);
	NODEOS_LEAVE;
	return NODEOS_EXTERN_ERROR(err);
}

/* an_cred_current is inlined */
