/*
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * Error handling functions
 */
#include <stdarg.h>  
#include <stdio.h>   /* vsnprintf */
#include <assert.h>

#include <nodeos/an_error.h>

/*
 * XXX the global error object.  Possibly should be thread-local, or
 * an explicit parameter to every function call.
 *
 * XXX these are not static so that a debugger can easily see them.
 */
ani_error_t ani_error_globalnum;
char ani_error_globalmsg[1024];

ani_error_t
ani_set_error_internal(int errcode, const char *file, int line,
		       const char *msg, ...)
{
	int off;
	va_list argptr;
	
	ani_error_globalnum = (ani_error_t)errcode;

	/* Insert the file and line info into ani_error_globalmsg */
	off = snprintf(ani_error_globalmsg, sizeof(ani_error_globalmsg),
		       "%s:%d: ", file, line);

	/* Print the user's message after the file:line info in the buffer. */
	va_start(argptr, msg);
	(void) vsnprintf(ani_error_globalmsg+off,
			 sizeof(ani_error_globalmsg)-off, msg, argptr);
	va_end(argptr);

	return (ani_error_t)errcode;
}

