/*
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

#include <stdio.h>
#include <signal.h>

#include <unistd.h>

#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/param.h>

#include <net/if.h>
#include <netinet/in.h>

#include <linux/if_packet.h>
#include <linux/if_ether.h>
#include <netinet/if_ether.h>

#include <sys/uio.h>

#include <nodeos/impl/an_error.h>
#include <nodeos/config.h>
#include "os.h"
#include "ani_debug.h"


/*
 * This file is empty if CHAN_ETH isn't defined.
 */
#ifdef CHAN_ETH

int
ani_os_open_eth(char *dev)
{
	int fd;
	struct ifreq ifr;
	struct sockaddr_ll sa;
	struct packet_mreq mr;

	DBG(DBG_DEVICE_INIT, ani_dprintf("opening device %s\n", dev));
	
	fd = socket(PF_PACKET, SOCK_RAW, htons(ETH_P_ALL));
	if (fd < 0) {
		/* print error */
		return -1;
	}

	memset(&sa, 0, sizeof(sa));
	strcpy(ifr.ifr_name, dev);

	if (ioctl(fd, SIOCGIFINDEX, &ifr) < 0) {
		/* print error */
		close(fd);
		return -1;
	}
	
	sa.sll_family = AF_PACKET;
	sa.sll_ifindex = ifr.ifr_ifindex;
	sa.sll_protocol = htons(ETH_P_ALL);

	if (bind(fd, (struct sockaddr *)&sa, sizeof(sa))) {
		/* print error */
		close(fd);
		return -1;
	}

	memset(&mr, 0, sizeof(mr));
	mr.mr_ifindex = sa.sll_ifindex;
	mr.mr_type = PACKET_MR_PROMISC;

	if (setsockopt(fd, SOL_PACKET, PACKET_ADD_MEMBERSHIP, (char *)&mr,
		       sizeof(mr)) < 0)
	{
		/* print error */
		close(fd);
		return -1;
	}

	return fd;
}

void
ani_os_close_eth(int fd)
{
	close(fd);
}

int
ani_os_read_packet(int fd, char *buf, int bufsize)
{
	struct sockaddr_ll from;
	int fromlen;
	ssize_t count;

	
  again:		
	fromlen = sizeof(from);
	count = recvfrom(fd, buf, bufsize, 0,
			 (struct sockaddr *)&from, &fromlen);

	if (count < 0 && errno == EINTR)
		goto again;

	return count;
}

/*
 * XXX assumes socket is connected
 */
int
ani_os_write_packet(int fd, char *buf, int bufsize)
{
	return (int)write(fd, buf, bufsize);
}
#endif /* CHAN_ETH */
