/*
 * Copyright (c) 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

#include <oskit/dev/dev.h> /* osenv_log() */
#include <stdarg.h>
#include <stdio.h>

#include "ani_debug.h"

/*
 * Replacement for the OSKit OSENV logging functions.  Puts all of it
 * under control of the Moab Debugging infrastructure.
 */
void
ani_oskit_init_log(void)
{
	/* nothing to do */
}

/*
 * All log-like messages generated by OSKit code come through
 * here.  They may (but usually don't) include newlines.
 *
 * Priority should be between 0 and 7.  '0' is for emergency messages,
 * 7 is for random crap. grep for OSENV_LOG_ in oskit/dev/dev.h
 */
void
osenv_vlog(int priority, const char *fmt, void *args)
{
	int doit;

#ifdef DEBUG
	/* Maximum verbosity is default when debugging. */
	doit = 1;
#else
	/* Non-debug builds, only show error messages. */
	doit = (priority < 3);
#endif

	DBG(DBG_OSKIT_LOG_NOTICE, doit = (priority < 5));
	DBG(DBG_OSKIT_LOG_ERR, doit = (priority < 3));
	DBG(DBG_OSKIT_LOG_NONE, doit = 0);

	if (doit) {
#ifdef DEBUG		
		ani_dvprintf(fmt, args);
#else
		vprintf(fmt, args);
#endif
	}
}

void
osenv_log(int priority, const char *fmt, ...)
{
	va_list args;

	va_start(args, fmt);
	osenv_vlog(priority, fmt, args);
	va_end(args);
}
