/*
 * Copyright (c) 2000 University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

#ifndef _PQ_H_
#define _PQ_H_

#ifdef OSKIT
#include <oskit/io/netio.h>
#define _SEND_TYPE_ oskit_netio_t *netio
#else
#define _SEND_TYPE_ int fd
#endif

/*
 * Define an opaque type to represent the internal data type.
 */
struct pq_scheduler {
};
typedef struct pq_scheduler pq_scheduler_t;

struct pq_client {
};
typedef struct pq_client pq_client_t;

/*
 * The maximum number of tickets a client should be given.  Something
 * to note is that ticket allocations like 1000:1:1:1:1 are not very
 * wise; stride scheduling has a large variance in such situations. If
 * it becomes ncessary to do allocations like that, we might need to
 * do hierarchical stride scheduling, which has much lower variance in
 * such situations.
 *
 * WARNING: This number is somewhat dependent on the value of STRIDE
 * in pq_impl.h. Probably want to look at that value before changing
 * this one.
 */
#define MAXTICKETS	(1 << 10)

/*
 * Prototypes
 */

/*
 * Create a packet scheduler. The scheduler is just a handle; packets
 * are actually sent to "clients" of the scheduler. The scheduler holds
 * the approriate goo for shipping packets out on the wire (ie: an oskit
 * netio). maxburst is the number of backlogged packets to send from the
 * watchdog timer. The new scheduler is returned in out_pq.
 */
int		pq_create_scheduler(_SEND_TYPE_,
				int maxburst, pq_scheduler_t **out_pq);

/*
 * Create a client of a scheduler. This is where packets are sent.
 * maxqsize is the size of the Q in which packets are held pending until
 * the scheduler ships them out. Tickets is the number of tickets that
 * the client should receive. The new client is returned in out_pq.
 */
int		pq_create_client(pq_scheduler_t *scheduler, int maxqsize,
				int tickets, pq_client_t **out_pq);

/*
 * Send a packet out. It would be nice if the representation did not
 * matter, but it really does; if its a bufio we need to worry about
 * reference counts and such. The size is provided so that we don't
 * have to query the packet to find that out later.
 *
 * The packet may or may not be shipped out right away.
 */
int		pq_send_packet(pq_client_t *client, void *packet, int size);

/*
 * Destroy a scheduler. It is very bad to destroy a scheduler before
 * all the children are destroyed, so in fact, it won't let you!
 */
int		pq_destroy_scheduler(pq_scheduler_t *scheduler);

/*
 * Destroy a client. Any packets that are queued up and ready to send
 * will be deallocated before the client is released.
 */
void		pq_destroy_client(pq_client_t *client);

/*
 * Modify the tickets for a client. If old_tickets is a non-null pointer, the
 * original ticket value is placed there.
 */
void		pq_modify_tickets(pq_client_t *client,
				int new_tickets, int *old_tickets);

#endif /* _PQ_H_ */
